@@ -145,20 +145,24 @@ dospinport(){
 
     copyfilelist $portfiles $APPDIR $TMPFILESDIR
 
+# use DOS line endings for text files for MSW archives.
     if [ $port = "msw" ]; then
         FILES=`find . -type f \( -path '*/CVS/*' -prune -o -exec ${SCRIPTDIR}/is_text.sh {} \; -print \)`
-        echo "$FILES" > /tmp/textfiles
+        
+    else
+        FILES=`find . -name \*.ds?`        
     fi
+    echo "$FILES" > /tmp/textfiles
 
     pushd /tmp/wx$port
-    # use DOS line endings for text files for MSW archives.
-    if [ $port = "msw" ]; then
-        pushd /tmp/wx$port/wx$portname-$VERSION
-        for file in `cat /tmp/textfiles`; do
-            unix2dos $file
-        done
-        popd
-    fi
+
+
+    pushd /tmp/wx$port/wx$portname-$VERSION
+    for file in `cat /tmp/textfiles`; do
+        unix2dos $file
+    done
+    popd
+
     echo "Creating wx$portname-$VERSION.zip..."
     zip $ZIPFLAGS -r -9 $APPDIR/deliver/wx$portname-$VERSION.zip .
     echo "Creating wx$portname-$VERSION.tar.gz..."
@@ -209,7 +213,7 @@ prepareforrelease()
     cp $APPDIR/docs/cocoa/install.txt $APPDIR/install-cocoa.txt
 
     # Now delete a few files that are unnecessary
-    rm -f BuildCVS.txt descrip.mms
+    rm -f BuildSVN.txt descrip.mms
     rm -f setup.h_vms
     rm -f docs/html/wxbook.htm docs/html/roadmap.htm
     rm -f -r contrib/docs/latex/ogl
