@@ -3,7 +3,7 @@
 ###############################################################################
 # Name:      is_text.sh
 # Purpose:   returns 0 if the file in cvs tree is text, 1 if binary
-# Version:   $Id: is_text.sh,v 1.4 2006/12/13 09:46:20 CE Exp $
+# Version:   $Id: is_text.sh 47922 2007-08-07 11:52:39Z CE $
 # Author:    VZ
 # Created:   2006-07-19
 # Copyright: (c) Vadim Zeitlin 2006 <vadim@wxwindows.org>
@@ -25,16 +25,9 @@ if [ `basename $1` = "config.guess" ] ; then
     exit 1
 fi
 
-entries=`dirname $1`/CVS/Entries
-if [ ! -f $entries ]; then
-    echo "CVS entries file \"$entries\" not found." >&2
-    exit 3
-fi
+if  svn proplist $1 | grep -q "eol-style"  ; then
+    exit 0
+fi    
 
-# we look for -kb in the last field, optionally followed by tag name
-re="^/`basename $1`/.*/-kb/\(T[^/]*\)\{0,1\}$"
-if grep -q "$re" $entries; then
-    exit 1
-fi
-
-exit 0
+#either not a svn file or not one with eol-style
+exit 3
