@@ -5,7 +5,7 @@
 // Modified by: Vadim Zeitlin on 13.05.99: complete refont of message handling,
 //              elimination of Default(), ...
 // Created:     01/02/97
-// RCS-ID:      $Id: window.h,v 1.170 2006/10/09 16:39:10 VZ Exp $
+// RCS-ID:      $Id: window.h 47962 2007-08-08 12:38:13Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -193,7 +193,7 @@ public:
     wxWindow *FindItemByHWND(WXHWND hWnd, bool controlOnly = false) const;
 
     // MSW only: true if this control is part of the main control
-    virtual bool ContainsHWND(WXHWND WXUNUSED(hWnd)) const { return false; };
+    virtual bool ContainsHWND(WXHWND WXUNUSED(hWnd)) const { return false; }
 
 #if wxUSE_TOOLTIPS
     // MSW only: true if this window or any of its children have a tooltip
@@ -219,6 +219,12 @@ public:
     // created yet)
     void MSWUpdateStyle(long flagsOld, long exflagsOld);
 
+#if wxABI_VERSION >= 20805
+    // Helper for getting an appropriate theme style for the application. Unnecessary in
+    // 2.9 and above.
+	wxBorder GetThemedBorderStyle() const;
+#endif
+
     // translate wxWidgets coords into Windows ones suitable to be passed to
     // ::CreateWindow()
     //
