@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: event.cpp,v 1.188 2006/11/05 21:22:08 RD Exp $
+// RCS-ID:      $Id: event.cpp 48332 2007-08-22 14:46:50Z DE $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -115,7 +115,11 @@ class wxEventTableEntryModule: public wxModule
 DECLARE_DYNAMIC_CLASS(wxEventTableEntryModule)
 public:
     wxEventTableEntryModule() {}
-    bool OnInit() { return true; }
+    bool OnInit()
+    {
+        wxEventHashTable::ReconstructAll();
+        return true;
+    }
     void OnExit()
     {
         wxEventHashTable::ClearAll();
@@ -352,7 +356,7 @@ wxEvent::wxEvent(int theId, wxEventType commandType )
 }
 
 wxEvent::wxEvent(const wxEvent &src)
-    : wxObject()
+    : wxObject(src)
     , m_eventObject(src.m_eventObject)
     , m_eventType(src.m_eventType)
     , m_timeStamp(src.m_timeStamp)
@@ -850,6 +854,23 @@ void wxEventHashTable::ClearAll()
     }
 }
 
+// Rebuild all tables if they were cleared by ClearAll
+void wxEventHashTable::ReconstructAll()
+{
+    wxEventHashTable* table = sm_first;
+    while (table)
+    {
+        // This will only be true if the event table was cleared.
+        // What we do here is basically what the constructor does.
+        if(table->m_eventTypeTable == NULL)
+        {
+            table->AllocEventTypeTable(EVENT_TYPE_TABLE_INIT_SIZE);
+            table->m_rebuildHash = true;
+        }
+        table = table->m_next;
+    }
+}
+
 bool wxEventHashTable::HandleEvent(wxEvent &event, wxEvtHandler *self)
 {
     if (m_rebuildHash)
@@ -1028,9 +1049,10 @@ wxEvtHandler::~wxEvtHandler()
 
     if (m_dynamicEvents)
     {
-        wxList::iterator it = m_dynamicEvents->begin(),
-                         en = m_dynamicEvents->end();
-        for (;it != en; ++it)
+        for ( wxList::iterator it = m_dynamicEvents->begin(),
+                               end = m_dynamicEvents->end();
+              it != end;
+              ++it )
         {
 #if WXWIN_COMPATIBILITY_EVENT_TYPES
             wxEventTableEntry *entry = (wxEventTableEntry*)*it;
