@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     07.04.98 (adapted from appconf.cpp)
-// RCS-ID:      $Id: fileconf.cpp,v 1.162 2007/01/18 22:38:19 VZ Exp $
+// RCS-ID:      $Id: fileconf.cpp 49525 2007-10-29 19:00:07Z VZ $
 // Copyright:   (c) 1997 Karsten Ballder   &  Vadim Zeitlin
 //                       Ballueder@usa.net     <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
@@ -242,6 +242,7 @@ public:
 
   // get the last line belonging to an entry/subgroup of this group
   wxFileConfigLineList *GetGroupLine();     // line which contains [group]
+                                            // may be NULL for "/" only
   wxFileConfigLineList *GetLastEntryLine(); // after which our subgroups start
   wxFileConfigLineList *GetLastGroupLine(); // after which the next group starts
 
@@ -439,7 +440,7 @@ wxFileConfig::wxFileConfig(const wxString& appName, const wxString& vendorName,
         m_strLocalFile = GetLocalFileName(GetAppName());
 #if defined(__UNIX__) && !defined(__VMS)
         if ( style & wxCONFIG_USE_SUBDIR )
-            m_strLocalFile << wxFILE_SEP_PATH << GetAppName();
+            m_strLocalFile << wxFILE_SEP_PATH << GetAppName() << _T(".conf");
 #endif
     }
 
@@ -492,16 +493,16 @@ wxFileConfig::wxFileConfig(wxInputStream &inStream, const wxMBConv& conv)
     m_linesHead =
     m_linesTail = NULL;
 
-    // translate everything to the current (platform-dependent) line
-    // termination character
-    wxString strTrans;
+    // read the entire stream contents in memory
+    wxString str;
     {
-        wxString strTmp;
+        static const size_t chunkLen = 1024;
 
-        char buf[1024];
+        wxMemoryBuffer buf(chunkLen);
         do
         {
-            inStream.Read(buf, WXSIZEOF(buf)-1);  // leave room for the NULL
+            inStream.Read(buf.GetAppendBuf(chunkLen), chunkLen);
+            buf.UngetAppendBuf(inStream.LastRead());
 
             const wxStreamError err = inStream.GetLastError();
 
@@ -510,19 +511,27 @@ wxFileConfig::wxFileConfig(wxInputStream &inStream, const wxMBConv& conv)
                 wxLogError(_("Error reading config options."));
                 break;
             }
-
-            // FIXME: this is broken because if we have part of multibyte
-            //        character in the buffer (and another part hasn't been
-            //        read yet) we're going to lose data because of conversion
-            //        errors
-            buf[inStream.LastRead()] = '\0';
-            strTmp += conv.cMB2WX(buf);
         }
         while ( !inStream.Eof() );
 
-        strTrans = wxTextBuffer::Translate(strTmp);
+#if wxUSE_UNICODE
+        size_t len;
+        str = conv.cMB2WC((char *)buf.GetData(), buf.GetDataLen(), &len);
+        if ( !len && buf.GetDataLen() )
+        {
+            wxLogError(_("Failed to read config options."));
+        }
+#else // !wxUSE_UNICODE
+        // no need for conversion
+        str.assign((char *)buf.GetData(), buf.GetDataLen());
+#endif // wxUSE_UNICODE/!wxUSE_UNICODE
     }
 
+
+    // translate everything to the current (platform-dependent) line
+    // termination character
+    str = wxTextBuffer::Translate(str);
+
     wxMemoryText memText;
 
     // Now we can add the text to the memory text. To do this we extract line
@@ -534,21 +543,21 @@ wxFileConfig::wxFileConfig(wxInputStream &inStream, const wxMBConv& conv)
     const wxChar *pEOL = wxTextBuffer::GetEOL(wxTextBuffer::typeDefault);
     const size_t EOLLen = wxStrlen(pEOL);
 
-    int posLineStart = strTrans.Find(pEOL);
+    int posLineStart = str.Find(pEOL);
     while ( posLineStart != -1 )
     {
-        wxString line(strTrans.Left(posLineStart));
+        wxString line(str.Left(posLineStart));
 
         memText.AddLine(line);
 
-        strTrans = strTrans.Mid(posLineStart + EOLLen);
+        str = str.Mid(posLineStart + EOLLen);
 
-        posLineStart = strTrans.Find(pEOL);
+        posLineStart = str.Find(pEOL);
     }
 
     // also add whatever we have left in the translated string.
-    if ( !strTrans.empty() )
-        memText.AddLine(strTrans);
+    if ( !str.empty() )
+        memText.AddLine(str);
 
     // Finally we can parse it all.
     Parse(memText, true /* local */);
@@ -598,16 +607,8 @@ void wxFileConfig::Parse(const wxTextBuffer& buffer, bool bLocal)
 
     // add the line to linked list
     if ( bLocal )
-    {
       LineListAppend(strLine);
 
-      // let the root group have its start line as well
-      if ( !n )
-      {
-        m_pCurrentGroup->SetLine(m_linesTail);
-      }
-    }
-
 
     // skip leading spaces
     for ( pStart = strLine; wxIsspace(*pStart); pStart++ )
@@ -990,8 +991,8 @@ bool wxFileConfig::DoWriteString(const wxString& key, const wxString& szValue)
 
         SetDirty();
 
-            // this will add a line for this group if it didn't have it before
-
+        // this will add a line for this group if it didn't have it before (or
+        // do nothing for the root but it's ok as it always exists anyhow)
         (void)m_pCurrentGroup->GetGroupLine();
     }
     else
@@ -1047,15 +1048,17 @@ bool wxFileConfig::Flush(bool /* bCurrentOnly */)
   }
 
   // write all strings to file
+  wxString filetext;
+  filetext.reserve(4096);
   for ( wxFileConfigLineList *p = m_linesHead; p != NULL; p = p->Next() )
   {
-    wxString line = p->Text();
-    line += wxTextFile::GetEOL();
-    if ( !file.Write(line, *m_conv) )
-    {
-      wxLogError(_("can't write user configuration file."));
-      return false;
-    }
+    filetext << p->Text() << wxTextFile::GetEOL();
+  }
+
+  if ( !file.Write(filetext, *m_conv) )
+  {
+    wxLogError(_("can't write user configuration file."));
+    return false;
   }
 
   if ( !file.Commit() )
@@ -1763,7 +1766,7 @@ bool wxFileConfigGroup::DeleteSubgroup(wxFileConfigGroup *pGroup)
 
             m_pLastGroup = NULL;
             for ( wxFileConfigLineList *pl = pLine->Prev();
-                  pl && pl != m_pLine && !m_pLastGroup;
+                  pl && !m_pLastGroup;
                   pl = pl->Prev() )
             {
                 // does this line belong to our subgroup?
@@ -1777,6 +1780,9 @@ bool wxFileConfigGroup::DeleteSubgroup(wxFileConfigGroup *pGroup)
                         break;
                     }
                 }
+
+                if ( pl == m_pLine )
+                    break;
             }
         }
 
@@ -1812,29 +1818,21 @@ bool wxFileConfigGroup::DeleteEntry(const wxChar *szName)
       // our last entry is being deleted - find the last one which stays
       wxASSERT( m_pLine != NULL );  // if we have an entry with !NULL pLine...
 
-      // go back until we find another entry or reach the group's line
+      // find the previous entry (if any)
       wxFileConfigEntry *pNewLast = NULL;
-      size_t n, nEntries = m_aEntries.Count();
-      wxFileConfigLineList *pl;
-      for ( pl = pLine->Prev(); pl != m_pLine; pl = pl->Prev() ) {
-        // is it our subgroup?
-        for ( n = 0; (pNewLast == NULL) && (n < nEntries); n++ ) {
-          if ( m_aEntries[n]->GetLine() == m_pLine )
-            pNewLast = m_aEntries[n];
-        }
-
-        if ( pNewLast != NULL ) // found?
+      const wxFileConfigLineList * const
+        pNewLastLine = m_pLastEntry->GetLine()->Prev();
+      const size_t nEntries = m_aEntries.GetCount();
+      for ( size_t n = 0; n < nEntries; n++ ) {
+        if ( m_aEntries[n]->GetLine() == pNewLastLine ) {
+          pNewLast = m_aEntries[n];
           break;
+        }
       }
 
-      if ( pl == m_pLine ) {
-        wxASSERT( !pNewLast );  // how comes it has the same line as we?
-
-        // we've reached the group line without finding any subgroups
-        m_pLastEntry = NULL;
-      }
-      else
-        m_pLastEntry = pNewLast;
+      // pNewLast can be NULL here -- it's ok and can happen if we have no
+      // entries left
+      m_pLastEntry = pNewLast;
     }
 
     m_pConfig->LineListRemove(pLine);
