@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     
-// RCS-ID:      $Id: graphcmn.cpp,v 1.30 2006/12/05 23:42:48 RD Exp $
+// RCS-ID:      $Id: graphcmn.cpp 49287 2007-10-21 11:52:54Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -579,12 +579,15 @@ void wxGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, w
 void wxGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, const wxGraphicsBrush& backgroundBrush ) 
 {
     wxGraphicsBrush formerBrush = m_brush;
+	wxGraphicsPen formerPen = m_pen;
     wxDouble width;
     wxDouble height;
     wxDouble descent;
     wxDouble externalLeading;
     GetTextExtent( str , &width, &height, &descent, &externalLeading );
     SetBrush( backgroundBrush );
+	// to make sure our 'OffsetToPixelBoundaries' doesn't move the fill shape
+	SetPen( wxNullGraphicsPen );
 
     wxGraphicsPath path = CreatePath();
     path.AddRectangle( x , y, width, height );
@@ -592,11 +595,13 @@ void wxGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, c
 
     DrawText( str, x ,y);
     SetBrush( formerBrush );
+	SetPen( formerPen );
 }
 
 void wxGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, wxDouble angle, const wxGraphicsBrush& backgroundBrush )
 {
     wxGraphicsBrush formerBrush = m_brush;
+	wxGraphicsPen formerPen = m_pen;
 
     wxDouble width;
     wxDouble height;
@@ -604,6 +609,8 @@ void wxGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, w
     wxDouble externalLeading;
     GetTextExtent( str , &width, &height, &descent, &externalLeading );
     SetBrush( backgroundBrush );
+	// to make sure our 'OffsetToPixelBoundaries' doesn't move the fill shape
+	SetPen( wxNullGraphicsPen );
 
     wxGraphicsPath path = CreatePath();
     path.MoveToPoint( x , y );
@@ -615,6 +622,7 @@ void wxGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, w
     FillPath( path );
     DrawText( str, x ,y, angle);
     SetBrush( formerBrush );
+	SetPen( formerPen );
 }
 
 void wxGraphicsContext::StrokeLine( wxDouble x1, wxDouble y1, wxDouble x2, wxDouble y2)
