@@ -5,7 +5,7 @@
 //              Dirk Holtwick, Ron Lee
 // Modified by: Ron Lee
 // Created:
-// RCS-ID:      $Id: sizer.cpp,v 1.152 2006/11/23 17:50:52 VZ Exp $
+// RCS-ID:      $Id: sizer.cpp 49676 2007-11-06 10:47:58Z JS $
 // Copyright:   (c) Robin Dunn, Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -460,6 +460,11 @@ bool wxSizerItem::IsShown() const
             // shown, so are we (this arbitrariness is the reason for
             // deprecating this function)
             {
+                // Some apps (such as dialog editors) depend on an empty sizer still
+                // being laid out correctly and reporting the correct size and position.
+                if (m_sizer->GetChildren().GetCount() == 0)
+                    return true;
+
                 for ( wxSizerItemList::compatibility_iterator
                         node = m_sizer->GetChildren().GetFirst();
                       node;
@@ -511,6 +516,9 @@ wxSizerItem* wxSizer::Insert( size_t index, wxSizerItem *item )
     if ( item->GetWindow() )
         item->GetWindow()->SetContainingSizer( this );
 
+    if ( item->GetSizer() )
+        item->GetSizer()->SetContainingWindow( m_containingWindow );
+
     return item;
 }
 
@@ -1469,7 +1477,9 @@ void wxFlexGridSizer::AdjustForFlexDirection()
         // and now fill it with the largest value
         for ( n = 0; n < count; ++n )
         {
-            array[n] = largest;
+            // don't touch hidden rows
+            if ( array[n] != -1 )
+                array[n] = largest;
         }
     }
 }
@@ -1509,9 +1519,7 @@ void wxFlexGridSizer::AdjustForGrowables(const wxSize& sz, const wxSize& minsz,
             {
                 if (m_growableRows[idx] >= nrows )
                     continue;
-                if (m_rowHeights[ m_growableRows[idx] ] == -1)
-                    m_rowHeights[ m_growableRows[idx] ] = 0;
-                else
+                if (m_rowHeights[ m_growableRows[idx] ] != -1)
                 {
                     int delta = (sz.y - minsz.y);
                     if (sum_proportions == 0)
@@ -1561,9 +1569,7 @@ void wxFlexGridSizer::AdjustForGrowables(const wxSize& sz, const wxSize& minsz,
             {
                 if (m_growableCols[idx] >= ncols )
                     continue;
-                if (m_colWidths[ m_growableCols[idx] ] == -1)
-                    m_colWidths[ m_growableCols[idx] ] = 0;
-                else
+                if (m_colWidths[ m_growableCols[idx] ] != -1)
                 {
                     int delta = (sz.x - minsz.x);
                     if (sum_proportions == 0)
@@ -1648,6 +1654,7 @@ void wxBoxSizer::RecalcSizes()
 
     wxPoint pt( m_position );
 
+    int stretchable = m_stretchable;
     wxSizerItemList::compatibility_iterator node = m_children.GetFirst();
     while (node)
     {
@@ -1664,7 +1671,9 @@ void wxBoxSizer::RecalcSizes()
                 {
                     // Because of at least one visible item has non-zero
                     // proportion then m_stretchable is not zero
-                    height = (delta * item->GetProportion()) / m_stretchable;
+                    height = (delta * item->GetProportion()) / stretchable;
+                    delta -= height;
+                    stretchable -= item->GetProportion();
                 }
 
                 wxPoint child_pos( pt );
@@ -1690,7 +1699,9 @@ void wxBoxSizer::RecalcSizes()
                 {
                     // Because of at least one visible item has non-zero
                     // proportion then m_stretchable is not zero
-                    width = (delta * item->GetProportion()) / m_stretchable;
+                    width = (delta * item->GetProportion()) / stretchable;
+                    delta -= width;
+                    stretchable -= item->GetProportion();
                 }
 
                 wxPoint child_pos( pt );
@@ -2020,7 +2031,8 @@ void wxStdDialogButtonSizer::Realize()
             if (m_buttonAffirmative->GetId() == wxID_SAVE){
                 // these buttons have set labels under Mac so we should use them
                 m_buttonAffirmative->SetLabel(_("Save"));
-                m_buttonNegative->SetLabel(_("Don't Save"));
+                if (m_buttonNegative)
+                    m_buttonNegative->SetLabel(_("Don't Save"));
             }
         }
 
