@@ -1,10 +1,9 @@
-/////////////////////////////////////////////////////////////////////////////
 // Name:        src/common/utilscmn.cpp
 // Purpose:     Miscellaneous utility functions and classes
 // Author:      Julian Smart
 // Modified by:
 // Created:     29/01/98
-// RCS-ID:      $Id: utilscmn.cpp,v 1.176 2006/10/30 11:46:44 RR Exp $
+// RCS-ID:      $Id: utilscmn.cpp 48211 2007-08-20 01:41:09Z KO $
 // Copyright:   (c) 1998 Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -647,24 +646,27 @@ static bool ReadAll(wxInputStream *is, wxArrayString& output)
 
     wxTextInputStream tis(*is);
 
-    bool cont = true;
-    while ( cont )
+    for ( ;; )
     {
         wxString line = tis.ReadLine();
-        if ( is->Eof() )
-            break;
 
+        // check for EOF before other errors as it's not really an error
+        if ( is->Eof() )
+        {
+            // add the last, possibly incomplete, line
+            if ( !line.empty() )
+                output.Add(line);
+            break;
+        }
+
+        // any other error is fatal
         if ( !*is )
-        {
-            cont = false;
-        }
-        else
-        {
-            output.Add(line);
-        }
+            return false;
+
+        output.Add(line);
     }
 
-    return cont;
+    return true;
 }
 #endif // wxUSE_STREAMS
 
@@ -722,18 +724,23 @@ long wxExecute(const wxString& command,
 // Launch default browser
 // ----------------------------------------------------------------------------
 
-bool wxLaunchDefaultBrowser(const wxString& urlOrig, int flags)
+#include "wx/private/browserhack28.h"
+
+static bool wxLaunchDefaultBrowserBaseImpl(const wxString& url, int flags);
+
+// Use wxLaunchDefaultBrowserBaseImpl by default
+static wxLaunchDefaultBrowserImpl_t s_launchBrowserImpl = &wxLaunchDefaultBrowserBaseImpl;
+
+// Function the GUI library can call to provide a better implementation
+WXDLLIMPEXP_BASE void wxSetLaunchDefaultBrowserImpl(wxLaunchDefaultBrowserImpl_t newImpl)
+{
+    s_launchBrowserImpl = newImpl!=NULL ? newImpl : &wxLaunchDefaultBrowserBaseImpl;
+}
+
+static bool wxLaunchDefaultBrowserBaseImpl(const wxString& url, int flags)
 {
     wxUnusedVar(flags);
 
-    // set the scheme of url to http if it does not have one
-    // RR: This doesn't work if the url is just a local path
-    wxString url(urlOrig);
-    wxURI uri(url);
-    if ( !uri.HasScheme() )
-        url.Prepend(wxT("http://"));
-
-
 #if defined(__WXMSW__)
 
 #if wxUSE_IPC
@@ -741,6 +748,7 @@ bool wxLaunchDefaultBrowser(const wxString& urlOrig, int flags)
     {
         // ShellExecuteEx() opens the URL in an existing window by default so
         // we can't use it if we need a new window
+        wxURI uri(url);
         wxRegKey key(wxRegKey::HKCR, uri.GetScheme() + _T("\\shell\\open"));
         if ( !key.Exists() )
         {
@@ -915,6 +923,25 @@ bool wxLaunchDefaultBrowser(const wxString& urlOrig, int flags)
     wxLogError(_T("No default application configured for HTML files."));
 
 #endif // !wxUSE_MIMETYPE && !__WXMSW__
+    return false;
+}
+
+bool wxLaunchDefaultBrowser(const wxString& urlOrig, int flags)
+{
+    // set the scheme of url to http if it does not have one
+    // RR: This doesn't work if the url is just a local path
+    wxString url(urlOrig);
+    wxURI uri(url);
+    if ( !uri.HasScheme() )
+    {
+        if (wxFileExists(urlOrig))
+            url.Prepend( wxT("file://") );
+        else
+            url.Prepend(wxT("http://"));
+    }
+
+    if(s_launchBrowserImpl(url, flags))
+        return true;
 
     wxLogSysError(_T("Failed to open URL \"%s\" in default browser."),
                   url.c_str());
