@@ -4,7 +4,7 @@
 // Author:      Ove Kaven
 // Modified by: Ron Lee, Francesco Montorsi
 // Created:     09/04/99
-// RCS-ID:      $Id: wxchar.cpp,v 1.137 2007/01/08 11:17:15 VZ Exp $
+// RCS-ID:      $Id: wxchar.cpp 49328 2007-10-22 11:32:59Z VZ $
 // Copyright:   (c) wxWidgets copyright
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -1199,7 +1199,11 @@ int WXDLLEXPORT wxVsnprintf_(wxChar *buf, size_t lenMax,
         return lenMax+1;     // not enough space in the output buffer !
     }
 
-    wxASSERT(lenCur == wxStrlen(buf));
+    // Don't do:
+    //      wxASSERT(lenCur == wxStrlen(buf));
+    // in fact if we embedded NULLs in the output buffer (using %c with a '\0')
+    // such check would fail
+
     return lenCur;
 }
 
@@ -1246,6 +1250,20 @@ int WXDLLEXPORT wxSnprintf_(wxChar *buf, size_t len, const wxChar *format, ...)
 
 #endif //__DMC__
 
+#if defined(__MINGW32__) && ( defined(_STLPORT_VERSION) && _STLPORT_VERSION >= 0x510 )
+    /* MinGW with STLPort 5.1 has clashing defines for _stprintf so use swprintf */
+    /* STLPort 5.1 defines standard (C99) vswprintf() and swprintf() that takes count. */
+    int wxSprintf (wchar_t * s, const wchar_t * format, ... )
+    {
+        va_list arglist;
+    
+        va_start( arglist, format );
+        int iLen = swprintf ( s, -1, format, arglist );
+        va_end( arglist );
+        return iLen ;
+    }
+#endif // MINGW32 _STLPORT_VERSION >= 0x510
+
 // ----------------------------------------------------------------------------
 // implement the standard IO functions for wide char if libc doesn't have them
 // ----------------------------------------------------------------------------
@@ -2021,7 +2039,7 @@ WXDLLEXPORT double wxStrtod(const wxChar *nptr, wxChar **endptr)
     while (wxIsdigit(*nptr)) nptr++;
   }
 
-  wxString data(nptr, nptr-start);
+  wxString data(start, nptr-start);
   wxWX2MBbuf dat = data.mb_str(wxConvLibc);
   char *rdat = wxMBSTRINGCAST dat;
   double ret = strtod(dat, &rdat);
@@ -2136,13 +2154,14 @@ wxChar * WXDLLEXPORT wxGetenv(const wxChar *name)
     return getenv(name);
 #endif
 }
+#endif // wxNEED_WX_STDLIB_H
 
+#ifdef wxNEED_WXSYSTEM
 int WXDLLEXPORT wxSystem(const wxChar *psz)
 {
     return system(wxConvLibc.cWX2MB(psz));
 }
-
-#endif // wxNEED_WX_STDLIB_H
+#endif // wxNEED_WXSYSTEM
 
 #ifdef wxNEED_WX_TIME_H
 WXDLLEXPORT size_t
