@@ -4,7 +4,7 @@
 // Author:      Karsten Ballder
 // Modified by:
 // Created:     09.05.1999
-// RCS-ID:      $Id: progdlgg.cpp,v 1.98 2006/10/22 21:08:04 VZ Exp $
+// RCS-ID:      $Id: progdlgg.cpp 49601 2007-11-02 16:11:08Z VZ $
 // Copyright:   (c) Karsten Ballder
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -176,7 +176,7 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
     m_display_estimated = m_last_timeupdate = m_break = 0;
     m_ctdelay = 0;
 
-    // if we are going to have at least one label, remmeber it in this var
+    // if we are going to have at least one label, remember it in this var
     wxStaticText *label = NULL;
 
     // also count how many labels we really have
@@ -261,9 +261,7 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
         sizeDlg.y += 2*LAYOUT_MARGIN;
 
         // try to make the dialog not square but rectangular of reasonable width
-        sizeDlg.x = (wxCoord)wxMax(widthText, 4*sizeDlg.y/3);
-        sizeDlg.x *= 3;
-        sizeDlg.x /= 2;
+        sizeDlg.x = (wxCoord)wxMax(3*widthText/2, 4*sizeDlg.y/3);
         SetClientSize(sizeDlg);
     }
 
@@ -338,12 +336,8 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
 
     wxASSERT_MSG( value <= m_maximum, wxT("invalid progress value") );
 
-    // fill up the gauge if value == maximum because this means that the dialog
-    // is going to close and the gauge shouldn't be partly empty in this case
-    if ( m_gauge && value <= m_maximum )
-    {
-        m_gauge->SetValue(value == m_maximum ? value : value + 1);
-    }
+    if ( m_gauge )
+        m_gauge->SetValue(value);
 
     UpdateMessage(newmsg);
 
