@@ -2,7 +2,7 @@
 // Name:        src/gtk/choice.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: choice.cpp,v 1.106 2006/10/31 08:49:52 RD Exp $
+// Id:          $Id: choice.cpp 48797 2007-09-19 13:48:25Z RR $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -118,8 +118,8 @@ bool wxChoice::Create( wxWindow *parent, wxWindowID id,
         m_strings = new wxSortedArrayString;
     }
 
-    // begin with no selection
-    m_selection_hack = wxNOT_FOUND;
+    // If we have items, GTK will choose the first item by default
+    m_selection_hack = n > 0 ? 0 : wxNOT_FOUND;
 
     GtkWidget *menu = gtk_menu_new();
 
@@ -382,6 +382,8 @@ void wxChoice::SetString(unsigned int n, const wxString& str)
 
             gtk_label_set_text( label, wxGTK_CONV( str ) );
 
+            InvalidateBestSize();
+
             return;
         }
         child = child->next;
