@@ -3,7 +3,7 @@
 // Author:      Robert Roebling
 // Purpose:     Implement GNOME printing support
 // Created:     09/20/04
-// RCS-ID:      $Id: gprint.cpp,v 1.53.2.1 2007/05/11 09:23:16 RR Exp $
+// RCS-ID:      $Id: gprint.cpp 48273 2007-08-21 10:53:06Z RR $
 // Copyright:   Robert Roebling
 // Licence:     wxWindows Licence
 /////////////////////////////////////////////////////////////////////////////
@@ -116,6 +116,11 @@ public:
     wxDL_METHOD_DEFINE( gint, gnome_print_grestore,
         (GnomePrintContext *pc), (pc), 0 )
 
+    wxDL_METHOD_DEFINE( gint, gnome_print_clip,
+        (GnomePrintContext *pc), (pc), 0 )
+    wxDL_METHOD_DEFINE( gint, gnome_print_eoclip,
+        (GnomePrintContext *pc), (pc), 0 )
+
     wxDL_METHOD_DEFINE( gint, gnome_print_beginpage,
         (GnomePrintContext *pc, const guchar* name), (pc, name), 0 )
     wxDL_METHOD_DEFINE( gint, gnome_print_showpage,
@@ -156,6 +161,9 @@ public:
         (GnomePrintConfig *config, const guchar *key, gboolean value), (config, key, value), false )
     wxDL_METHOD_DEFINE( gboolean, gnome_print_config_set_length,
         (GnomePrintConfig *config, const guchar *key, gdouble value, const GnomePrintUnit *unit), (config, key, value, unit), false )
+        
+    wxDL_METHOD_DEFINE( guchar*, gnome_print_config_get,
+        (GnomePrintConfig *config, const guchar *key), (config, key), NULL )
     wxDL_METHOD_DEFINE( gboolean, gnome_print_config_get_length,
         (GnomePrintConfig *config, const guchar *key, gdouble *val, const GnomePrintUnit **unit), (config, key, val, unit), false )
 
@@ -242,6 +250,9 @@ void wxGnomePrintLibrary::InitializeMethods()
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_gsave, success )
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_grestore, success )
 
+    wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_clip, success )
+    wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_eoclip, success )
+
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_beginpage, success )
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_showpage, success )
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_end_doc, success )
@@ -264,6 +275,8 @@ void wxGnomePrintLibrary::InitializeMethods()
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_config_set_double, success )
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_config_set_int, success )
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_config_set_length, success )
+    
+    wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_config_get, success )
     wxDL_METHOD_LOAD( m_gnome_print_lib, gnome_print_config_get_length, success )
 
     wxDL_METHOD_LOAD( m_gnome_printui_lib, gnome_print_dialog_new, success )
@@ -301,7 +314,13 @@ wxGnomePrintNativeData::~wxGnomePrintNativeData()
 
 bool wxGnomePrintNativeData::TransferTo( wxPrintData &data )
 {
-    // TODO
+    guchar *res = gs_lgp->gnome_print_config_get( m_config,
+            (guchar*)(char*)GNOME_PRINT_KEY_PAGE_ORIENTATION );
+    if (g_ascii_strcasecmp((const gchar *)res,"R90") == 0)
+        data.SetOrientation( wxLANDSCAPE );
+    else
+        data.SetOrientation( wxPORTRAIT );
+    g_free( res );
     
     return true;
 }
@@ -344,41 +363,33 @@ bool wxGnomePrintNativeData::TransferFrom( const wxPrintData &data )
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
                                     (guchar*)(char*)"A5" );
                                 break;
+        case wxPAPER_B4:        gs_lgp->gnome_print_config_set( m_config,
+                                    (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
+                                    (guchar*)(char*)"B4" );
+                                break;
         case wxPAPER_B5:        gs_lgp->gnome_print_config_set( m_config,
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"B5 (JIS)" );
+                                    (guchar*)(char*)"B5" );
                                 break;
         case wxPAPER_LETTER:        gs_lgp->gnome_print_config_set( m_config,
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Letter" );
+                                    (guchar*)(char*)"USLetter" );
                                 break;
         case wxPAPER_LEGAL:     gs_lgp->gnome_print_config_set( m_config,
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Legal" );
+                                    (guchar*)(char*)"USLegal" );
                                 break;
         case wxPAPER_EXECUTIVE: gs_lgp->gnome_print_config_set( m_config,
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
                                     (guchar*)(char*)"Executive" );
                                 break;
-        case wxPAPER_ENV_10:    gs_lgp->gnome_print_config_set( m_config,
-                                    (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Envelope #10" );
-                                break;
         case wxPAPER_ENV_C5:    gs_lgp->gnome_print_config_set( m_config,
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Envelope C5" );
+                                    (guchar*)(char*)"C5" );
                                 break;
         case wxPAPER_ENV_C6:    gs_lgp->gnome_print_config_set( m_config,
                                     (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Envelope C6" );
-                                break;
-        case wxPAPER_ENV_B5:    gs_lgp->gnome_print_config_set( m_config,
-                                    (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Envelope B5" );
-                                break;
-        case wxPAPER_ENV_MONARCH:    gs_lgp->gnome_print_config_set( m_config,
-                                    (guchar*)(char*)GNOME_PRINT_KEY_PAPER_SIZE,
-                                    (guchar*)(char*)"Envelope Monarch" );
+                                    (guchar*)(char*)"C6" );
                                 break;
         case wxPAPER_NONE:      break;
         
@@ -562,6 +573,8 @@ int wxGnomePrintDialog::ShowModal()
         return wxID_CANCEL;
     }
 
+    m_printDialogData.GetPrintData().ConvertFromNative();
+
     gint copies = 1;
     gboolean collate = false;
     gs_lgp->gnome_print_dialog_get_copies( (GnomePrintDialog*) m_widget, &copies, &collate );
@@ -634,11 +647,31 @@ wxGnomePageSetupDialog::wxGnomePageSetupDialog( wxWindow *parent,
     wxGnomePrintNativeData *native =
       (wxGnomePrintNativeData*) m_pageDialogData.GetPrintData().GetNativeData();
 
-    // This is required as the page setup dialog
+    // This *was* required as the page setup dialog
     // calculates wrong values otherwise.
+#if 0
     gs_lgp->gnome_print_config_set( native->GetPrintConfig(),
                             (const guchar*) GNOME_PRINT_KEY_PREFERED_UNIT,
                             (const guchar*) "Pts" );
+#endif
+
+    GnomePrintConfig *config = native->GetPrintConfig();
+
+    const GnomePrintUnit *mm_unit = gs_lgp->gnome_print_unit_get_by_abbreviation( (const guchar*) "mm" );
+
+    double ml = (double) m_pageDialogData.GetMarginTopLeft().x;
+    double mt = (double) m_pageDialogData.GetMarginTopLeft().y;
+    double mr = (double) m_pageDialogData.GetMarginBottomRight().x;
+    double mb = (double) m_pageDialogData.GetMarginBottomRight().y;
+    
+    gs_lgp->gnome_print_config_set_length (config,
+            (const guchar*) GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, ml, mm_unit );
+    gs_lgp->gnome_print_config_set_length (config,
+            (const guchar*) GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, mr, mm_unit );
+    gs_lgp->gnome_print_config_set_length (config,
+            (const guchar*) GNOME_PRINT_KEY_PAGE_MARGIN_TOP, mt, mm_unit );
+    gs_lgp->gnome_print_config_set_length (config,
+            (const guchar*) GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, mb, mm_unit );
 
     m_widget = gtk_dialog_new();
 
@@ -675,19 +708,18 @@ int wxGnomePageSetupDialog::ShowModal()
 {
     wxGnomePrintNativeData *native =
       (wxGnomePrintNativeData*) m_pageDialogData.GetPrintData().GetNativeData();
+      
     GnomePrintConfig *config = native->GetPrintConfig();
 
-    // Transfer data from m_pageDialogData to native dialog
 
     int ret = gtk_dialog_run( GTK_DIALOG(m_widget) );
 
     if (ret == GTK_RESPONSE_OK)
     {
         // Transfer data back to m_pageDialogData
+        m_pageDialogData.GetPrintData().ConvertFromNative();
 
         // I don't know how querying the last parameter works
-        // I cannot test it as the dialog is currently broken
-        // anyways (it only works for points).
         double ml,mr,mt,mb,pw,ph;
         gs_lgp->gnome_print_config_get_length (config,
             (const guchar*) GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &ml, NULL);
@@ -702,9 +734,8 @@ int wxGnomePageSetupDialog::ShowModal()
         gs_lgp->gnome_print_config_get_length (config,
             (const guchar*) GNOME_PRINT_KEY_PAPER_HEIGHT, &ph, NULL);
 
-        // This probably assumes that the user entered the
-        // values in Pts. Since that is the only the dialog
-        // works right now, we need to fix this later.
+        // This code converts correctly from what the user chose
+        // as the unit although I query Pts here
         const GnomePrintUnit *mm_unit = gs_lgp->gnome_print_unit_get_by_abbreviation( (const guchar*) "mm" );
         const GnomePrintUnit *pts_unit = gs_lgp->gnome_print_unit_get_by_abbreviation( (const guchar*) "Pts" );
         gs_lgp->gnome_print_convert_distance( &ml, pts_unit, mm_unit );
@@ -719,13 +750,6 @@ int wxGnomePageSetupDialog::ShowModal()
 
         m_pageDialogData.SetPaperSize( wxSize( (int)(pw+0.5), (int)(ph+0.5) ) );
 
-#if 0
-        wxPrintf( wxT("paper %d %d, top margin %d\n"),
-            m_pageDialogData.GetPaperSize().x,
-            m_pageDialogData.GetPaperSize().y,
-            m_pageDialogData.GetMarginTopLeft().x );
-#endif
-
         ret = wxID_OK;
     }
     else
@@ -763,7 +787,6 @@ IMPLEMENT_CLASS(wxGnomePrinter, wxPrinterBase)
 wxGnomePrinter::wxGnomePrinter( wxPrintDialogData *data ) :
     wxPrinterBase( data )
 {
-    m_gpc = NULL;
     m_native_preview = false;
 }
 
@@ -780,11 +803,11 @@ bool wxGnomePrinter::Print(wxWindow *parent, wxPrintout *printout, bool prompt )
     }
 
     wxPrintData printdata = GetPrintDialogData().GetPrintData();
+    
     wxGnomePrintNativeData *native =
         (wxGnomePrintNativeData*) printdata.GetNativeData();
 
     GnomePrintJob *job = gs_lgp->gnome_print_job_new( native->GetPrintConfig() );
-    m_gpc = gs_lgp->gnome_print_job_get_context (job);
 
     // The GnomePrintJob is temporarily stored in the
     // native print data as the native print dialog
@@ -803,7 +826,7 @@ bool wxGnomePrinter::Print(wxWindow *parent, wxPrintout *printout, bool prompt )
     if (prompt)
         dc = PrintDialog( parent );
     else
-        dc = new wxGnomePrintDC( this );
+        dc = new wxGnomePrintDC( printdata );
 
     if (m_native_preview)
         printout->SetIsPreview(true);
@@ -921,7 +944,7 @@ wxDC* wxGnomePrinter::PrintDialog( wxWindow *parent )
     m_native_preview = ret == wxID_PREVIEW;
 
     m_printDialogData = dialog.GetPrintDialogData();
-    return new wxGnomePrintDC( this );
+    return new wxGnomePrintDC( m_printDialogData.GetPrintData() );
 }
 
 bool wxGnomePrinter::Setup( wxWindow *parent )
@@ -935,38 +958,15 @@ bool wxGnomePrinter::Setup( wxWindow *parent )
 
 IMPLEMENT_CLASS(wxGnomePrintDC, wxDC)
 
-wxGnomePrintDC::wxGnomePrintDC( wxGnomePrinter *printer )
-{
-    m_printer = printer;
-
-    m_gpc = printer->GetPrintContext();
-    m_job = NULL; // only used and destroyed when created with wxPrintData
-
-    m_layout = gs_lgp->gnome_print_pango_create_layout( m_gpc );
-    m_fontdesc = pango_font_description_from_string( "Sans 12" );
-    m_context = NULL;
-
-    m_currentRed = 0;
-    m_currentBlue = 0;
-    m_currentGreen = 0;
-
-    m_signX =  1;  // default x-axis left to right
-    m_signY = -1;  // default y-axis bottom up -> top down
-    
-    GetSize( NULL, &m_deviceOffsetY );
-}
-
 wxGnomePrintDC::wxGnomePrintDC( const wxPrintData& data )
 {
-    m_printer = NULL;
     m_printData = data;
 
     wxGnomePrintNativeData *native =
         (wxGnomePrintNativeData*) m_printData.GetNativeData();
 
-    GnomePrintJob *job = gs_lgp->gnome_print_job_new( native->GetPrintConfig() );
-    m_gpc = gs_lgp->gnome_print_job_get_context (job);
-    m_job = job; // only used and destroyed when created with wxPrintData
+    m_job = native->GetPrintJob();
+    m_gpc = gs_lgp->gnome_print_job_get_context (m_job);
 
     m_layout = gs_lgp->gnome_print_pango_create_layout( m_gpc );
     m_fontdesc = pango_font_description_from_string( "Sans 12" );
@@ -984,8 +984,6 @@ wxGnomePrintDC::wxGnomePrintDC( const wxPrintData& data )
 
 wxGnomePrintDC::~wxGnomePrintDC()
 {
-    if (m_job)
-        g_object_unref (m_job);
 }
 
 bool wxGnomePrintDC::IsOk() const
@@ -1304,7 +1302,7 @@ void wxGnomePrintDC::DoDrawRoundedRectangle(wxCoord x, wxCoord y, wxCoord width,
 void wxGnomePrintDC::makeEllipticalPath(wxCoord x, wxCoord y,
                                         wxCoord width, wxCoord height)
 {
-    double r = 4 * (sqrt (2) - 1) / 3;
+    double r = 4 * (sqrt(2.) - 1) / 3;
     double  halfW = 0.5 * width,
             halfH = 0.5 * height,
             halfWR = r * halfW,
@@ -1764,10 +1762,32 @@ void wxGnomePrintDC::SetBackground( const wxBrush& brush )
 
 void wxGnomePrintDC::DoSetClippingRegion(wxCoord x, wxCoord y, wxCoord width, wxCoord height)
 {
+    wxDC::DoSetClippingRegion( x, y, width, height );
+    
+    gs_lgp->gnome_print_gsave( m_gpc );
+    
+    gs_lgp->gnome_print_newpath( m_gpc );
+    gs_lgp->gnome_print_moveto( m_gpc, XLOG2DEV(x), YLOG2DEV(y) );
+    gs_lgp->gnome_print_lineto( m_gpc, XLOG2DEV(x + width), YLOG2DEV(y) );
+    gs_lgp->gnome_print_lineto( m_gpc, XLOG2DEV(x + width), YLOG2DEV(y + height) );
+    gs_lgp->gnome_print_lineto( m_gpc, XLOG2DEV(x), YLOG2DEV(y + height) );
+    gs_lgp->gnome_print_closepath( m_gpc );
+    gs_lgp->gnome_print_clip( m_gpc );
 }
 
 void wxGnomePrintDC::DestroyClippingRegion()
 {
+    wxDC::DestroyClippingRegion();
+
+    gs_lgp->gnome_print_grestore( m_gpc );
+    
+#if 0
+    // not needed, we set the values in each
+    // drawing method anyways
+    SetPen( m_pen );
+    SetBrush( m_brush );
+    SetFont( m_font );
+#endif
 }
 
 bool wxGnomePrintDC::StartDoc(const wxString& message)
@@ -1829,10 +1849,6 @@ void wxGnomePrintDC::DoGetTextExtent(const wxString& string, wxCoord *width, wxC
         return;
     }
 
-    // Set new font description
-    if (theFont)
-        pango_layout_set_font_description( m_layout, theFont->GetNativeFontInfo()->description );
-
     // Set layout's text
 #if wxUSE_UNICODE
     const wxCharBuffer data = wxConvUTF8.cWC2MB( string );
@@ -1855,15 +1871,27 @@ void wxGnomePrintDC::DoGetTextExtent(const wxString& string, wxCoord *width, wxC
         return;
     }
 
+    PangoFontDescription *desc = (theFont) ? theFont->GetNativeFontInfo()->description : m_fontdesc;
+
+    gint oldSize = pango_font_description_get_size( desc );
+    double size = oldSize;
+    size = size * m_scaleY;
+    pango_font_description_set_size( desc, (gint)size );
+
+    // apply scaled font
+    pango_layout_set_font_description( m_layout, desc );
+
     pango_layout_set_text( m_layout, dataUTF8, strlen(dataUTF8) );
 
-    int w,h;
+    int w, h;
     pango_layout_get_pixel_size( m_layout, &w, &h );
 
+
     if (width)
         *width = (wxCoord)(w / m_scaleX);
     if (height)
         *height = (wxCoord)(h / m_scaleY);
+
     if (descent)
     {
         PangoLayoutIter *iter = pango_layout_get_iter(m_layout);
@@ -1872,9 +1900,11 @@ void wxGnomePrintDC::DoGetTextExtent(const wxString& string, wxCoord *width, wxC
         *descent = h - PANGO_PIXELS(baseline);
     }
 
-    // Reset old font description
-    if (theFont)
-        pango_layout_set_font_description( m_layout, m_fontdesc );
+    // reset unscaled size
+    pango_font_description_set_size( desc, oldSize );
+
+    // reset unscaled font
+    pango_layout_set_font_description( m_layout, m_fontdesc );
 }
 
 void wxGnomePrintDC::DoGetSize(int* width, int* height) const
@@ -1938,6 +1968,16 @@ void wxGnomePrintDC::SetDeviceOrigin( wxCoord x, wxCoord y )
     wxDC::SetDeviceOrigin( x, y );
 }
 
+void wxGnomePrintDC::SetPrintData(const wxPrintData& data)
+{ 
+    m_printData = data;
+    
+    if (m_printData.GetOrientation() == wxPORTRAIT)
+        GetSize( NULL, &m_deviceOffsetY );
+    else
+        GetSize( &m_deviceOffsetY, NULL );
+}
+
 void wxGnomePrintDC::SetResolution(int ppi)
 {
 }
