@@ -3,7 +3,7 @@
 // Purpose:
 // Author:      Robert Roebling
 // Modified by: Mart Raudsepp (GetMetric)
-// Id:          $Id: settings.cpp,v 1.69 2006/10/15 01:10:01 PC Exp $
+// Id:          $Id: settings.cpp 49665 2007-11-05 22:48:57Z JS $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -42,6 +42,7 @@ struct wxSystemObjects
              m_colHighlight,
              m_colHighlightText,
              m_colListBox,
+             m_colWindow,
              m_colBtnText,
              m_colMenuItemHighlight,
              m_colTooltip,
@@ -52,6 +53,21 @@ struct wxSystemObjects
 
 static wxSystemObjects gs_objects;
 
+void wxClearGtkSystemObjects()
+{
+    gs_objects.m_colBtnFace = wxColour();
+    gs_objects.m_colBtnShadow = wxColour();
+    gs_objects.m_colBtnHighlight = wxColour();
+    gs_objects.m_colHighlightText = wxColour();
+    gs_objects.m_colListBox = wxColour();
+    gs_objects.m_colWindow = wxColour();
+    gs_objects.m_colBtnText = wxColour();
+    gs_objects.m_colMenuItemHighlight = wxColour();
+    gs_objects.m_colTooltip = wxColour();
+    gs_objects.m_colTooltipText = wxColour();
+    gs_objects.m_fontSystem = wxNullFont;
+}
+
 // ----------------------------------------------------------------------------
 // wxSystemSettings implementation
 // ----------------------------------------------------------------------------
@@ -61,7 +77,8 @@ enum wxGtkWidgetType
 {
     wxGTK_BUTTON,
     wxGTK_LIST,
-    wxGTK_MENUITEM
+    wxGTK_MENUITEM,
+    wxGTK_TEXTCTRL
 };
 
 // the colour we need
@@ -90,6 +107,10 @@ static bool GetColourFromGTKWidget(GdkColor& gdkColor,
             widget = gtk_button_new();
             break;
 
+        case wxGTK_TEXTCTRL:
+            widget = gtk_text_view_new();
+            break;
+
         case wxGTK_LIST:
             widget = gtk_tree_view_new_with_model(
                 (GtkTreeModel*)gtk_list_store_new(1, G_TYPE_INT));
@@ -126,28 +147,26 @@ static bool GetColourFromGTKWidget(GdkColor& gdkColor,
         }
     }
 
-    gtk_widget_destroy( widget );
+    gtk_object_sink((GtkObject*)widget);
 
     return ok;
 }
 
 static void GetTooltipColors()
 {
-    GtkTooltips* tooltips = gtk_tooltips_new();
-    gtk_tooltips_force_window(tooltips);
-    gtk_widget_ensure_style(tooltips->tip_window);
-    GdkColor c = tooltips->tip_window->style->bg[GTK_STATE_NORMAL];
+    GtkWidget* widget = gtk_window_new(GTK_WINDOW_POPUP);
+    const char* name = "gtk-tooltip";
+    if (gtk_check_version(2, 11, 0))
+        name = "gtk-tooltips";
+    gtk_widget_set_name(widget, name);
+    gtk_widget_ensure_style(widget);
+
+    GdkColor c = widget->style->bg[GTK_STATE_NORMAL];
     gs_objects.m_colTooltip = wxColor(c);
-    c = tooltips->tip_window->style->fg[GTK_STATE_NORMAL];
+    c = widget->style->fg[GTK_STATE_NORMAL];
     gs_objects.m_colTooltipText = wxColor(c);
-#if GTK_CHECK_VERSION(2, 9, 0)
-    if (gtk_check_version(2, 9, 0) == NULL)
-        g_object_ref_sink(tooltips);
-    else
-#endif
-    {
-        gtk_object_sink((GtkObject*)tooltips);
-    }
+
+    gtk_widget_destroy(widget);
 }
 
 wxColour wxSystemSettingsNative::GetColour( wxSystemColour index )
@@ -178,7 +197,15 @@ wxColour wxSystemSettingsNative::GetColour( wxSystemColour index )
             break;
 
         case wxSYS_COLOUR_WINDOW:
-            color = *wxWHITE;
+            if (!gs_objects.m_colWindow.Ok())
+            {
+                gdkColor.red =
+                gdkColor.green =
+                gdkColor.blue = 0xFFFF;
+                GetColourFromGTKWidget(gdkColor, wxGTK_TEXTCTRL, GTK_STATE_NORMAL, wxGTK_BASE);
+                gs_objects.m_colWindow = wxColor(gdkColor);
+            }
+            color = gs_objects.m_colWindow;
             break;
 
         case wxSYS_COLOUR_3DDKSHADOW:
@@ -355,7 +382,7 @@ wxFont wxSystemSettingsNative::GetFont( wxSystemFont index )
                         gs_objects.m_fontSystem = wxFont(wxString::FromAscii(font_name));
                     g_free (font_name);
                 }
-                gtk_widget_destroy( widget );
+                gtk_object_sink((GtkObject*)widget);
             }
             font = gs_objects.m_fontSystem;
             break;
@@ -448,10 +475,10 @@ int wxSystemSettingsNative::GetMetric( wxSystemMetric index, wxWindow* win )
                                 case wxSYS_BORDER_X:
                                 case wxSYS_EDGE_X:
                                 case wxSYS_FRAMESIZE_X:
-                                    border_return = int(data[1]); // width of right extent
+                                    border_return = ((long*)data)[1]; // width of right extent
                                     break;
                                 default:
-                                    border_return = int(data[3]); // height of bottom extent
+                                    border_return = ((long*)data)[3]; // height of bottom extent
                                     break;
                             }
                         }
@@ -516,7 +543,7 @@ int wxSystemSettingsNative::GetMetric( wxSystemMetric index, wxWindow* win )
             // since that is what the API wants. But the values
             // are much bigger under GNOME than under Windows and
             // just seem to much in many cases to be useful.
-            // drag_threshold *= 2;   
+            // drag_threshold *= 2;
 
             return drag_threshold;
 
@@ -578,7 +605,7 @@ int wxSystemSettingsNative::GetMetric( wxSystemMetric index, wxWindow* win )
 
                 if ((type == XA_CARDINAL) && (format == 32) && (nitems >= 3) && (data))
                 {
-                    caption_height = int(data[2]); // top frame extent
+                    caption_height = ((long*)data)[2]; // top frame extent
                 }
 
                 if (data)
