@@ -2,7 +2,7 @@
 // Name:        src/gtk/toplevel.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: toplevel.cpp,v 1.140.4.2 2007/04/27 18:19:59 RD Exp $
+// Id:          $Id: toplevel.cpp 49971 2007-11-15 16:27:43Z MR $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -26,11 +26,16 @@
 
 #ifndef WX_PRECOMP
     #include "wx/frame.h"
+    #include "wx/menu.h"
     #include "wx/icon.h"
     #include "wx/log.h"
     #include "wx/app.h"
 #endif
 
+#if wxUSE_TOOLBAR
+#include "wx/toolbar.h"
+#endif
+
 #include "wx/gtk/private.h"
 #include "wx/evtloop.h"
 
@@ -709,11 +714,99 @@ bool wxTopLevelWindowGTK::EnableCloseButton( bool enable )
     return true;
 }
 
+// Helper for wxCreateAcceleratorTableForMenuBar
+static void wxAddAccelerators(wxList& accelEntries, wxMenu* menu)
+{
+    size_t i;
+    for (i = 0; i < menu->GetMenuItems().GetCount(); i++)
+    {
+        wxMenuItem* item = (wxMenuItem*) menu->GetMenuItems().Item(i)->GetData();
+        if (item->GetSubMenu())
+        {
+            wxAddAccelerators(accelEntries, item->GetSubMenu());
+        }
+        else if (!item->GetItemLabel().IsEmpty())
+        {
+            wxAcceleratorEntry* entry = wxAcceleratorEntry::Create(item->GetItemLabel());
+            if (entry)
+            {
+                entry->Set(entry->GetFlags(), entry->GetKeyCode(), item->GetId());
+                accelEntries.Append((wxObject*) entry);
+            }
+        }
+    }
+}
+
+// Create an accelerator table consisting of all the accelerators
+// from the menubar in the given menus
+static wxAcceleratorTable wxCreateAcceleratorTableForMenuBar(wxMenuBar* menuBar)
+{
+    wxList accelEntries;
+
+    size_t i;
+    for (i = 0; i < menuBar->GetMenuCount(); i++)
+    {
+        wxAddAccelerators(accelEntries, menuBar->GetMenu(i));
+    }
+
+    size_t n = accelEntries.GetCount();
+
+    if (n == 0)
+        return wxAcceleratorTable();
+
+    wxAcceleratorEntry* entries = new wxAcceleratorEntry[n];
+
+    for (i = 0; i < accelEntries.GetCount(); i++)
+    {
+        wxAcceleratorEntry* entry = (wxAcceleratorEntry*) accelEntries.Item(i)->GetData();
+        entries[i] = (*entry);
+        delete entry;
+        
+    }
+
+    wxAcceleratorTable table(n, entries);
+    delete[] entries;
+
+    return table;
+}
+
 bool wxTopLevelWindowGTK::ShowFullScreen(bool show, long style )
 {
     if (show == m_fsIsShowing)
         return false; // return what?
 
+    if (show)
+    {
+        // Preserve menubar accelerators during full-screen operation
+        wxFrame* frame = wxDynamicCast(this, wxFrame);
+        if (frame)
+        {
+            if (frame->GetMenuBar())
+            {
+                wxAcceleratorTable table(wxCreateAcceleratorTableForMenuBar(frame->GetMenuBar()));
+                if (table.IsOk())
+                    SetAcceleratorTable(table);
+            }
+#if wxUSE_TOOLBAR
+            if (frame->GetToolBar() && frame->GetToolBar()->IsShown())
+            {
+                frame->GetToolBar()->Show(false);
+            }
+#endif
+        }
+    }
+#if wxUSE_TOOLBAR
+    else
+    {
+        // FIXME: we need to remember whether the toolbar was previously hidden
+        wxFrame* frame = wxDynamicCast(this, wxFrame);
+        if (frame && frame->GetToolBar())
+        {
+            frame->GetToolBar()->Show(true);
+	}
+    }
+#endif
+
     m_fsIsShowing = show;
 
     wxX11FullScreenMethod method =
@@ -855,9 +948,6 @@ void wxTopLevelWindowGTK::DoSetSize( int x, int y, int width, int height, int si
 {
     wxASSERT_MSG( (m_widget != NULL), wxT("invalid frame") );
 
-    // this shouldn't happen: wxFrame, wxMDIParentFrame and wxMDIChildFrame have m_wxwindow
-    wxASSERT_MSG( (m_wxwindow != NULL), wxT("invalid frame") );
-
     // avoid recursions
     if (m_resizing)
         return;
@@ -1372,15 +1462,16 @@ bool wxTopLevelWindowGTK::SetTransparent(wxByte alpha)
     // from which the opacity property is checked from.
     Window win = wxGetTopmostWindowX11(dpy, GDK_WINDOW_XID (m_widget->window));
 
-    unsigned int opacity = alpha * 0x1010101;
-
     // Using pure Xlib to not have a GTK version check mess due to gtk2.0 not having GdkDisplay
     if (alpha == 0xff)
         XDeleteProperty(dpy, win, XInternAtom(dpy, "_NET_WM_WINDOW_OPACITY", False));
     else
+    {
+        long opacity = alpha * 0x1010101L;
         XChangeProperty(dpy, win, XInternAtom(dpy, "_NET_WM_WINDOW_OPACITY", False),
                         XA_CARDINAL, 32, PropModeReplace,
                         (unsigned char *) &opacity, 1L);
+    }
     XSync(dpy, False);
     return true;
 }
