@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: app.cpp,v 1.228.4.2 2007/04/21 23:17:26 KO Exp $
+// RCS-ID:      $Id: app.cpp 49620 2007-11-04 14:06:59Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -321,18 +321,25 @@ void wxApp::MacReopenApp()
     else
     {
         wxTopLevelWindow* firstIconized = NULL ;
+        wxTopLevelWindow* firstHidden = NULL ;
         while (node)
         {
             wxTopLevelWindow* win = (wxTopLevelWindow*) node->GetData();
-            if ( !win->IsIconized() )
+            if ( !win->IsShown() )
             {
-                firstIconized = NULL ;
-                break ;
+                // make sure we don't show 'virtual toplevel windows' like wxTaskBarIconWindow
+                if ( firstHidden == NULL && ( wxDynamicCast( win, wxFrame ) || wxDynamicCast( win, wxDialog ) ) )
+                   firstHidden = win ;
+            } 
+            else if ( win->IsIconized() )
+            { 
+                if ( firstIconized == NULL )
+                    firstIconized = win ;
             }
             else
             {
-                if ( firstIconized == NULL )
-                    firstIconized = win ;
+                // we do have a visible, non-iconized toplevelwindow -> do nothing
+                return;
             }
 
             node = node->GetNext();
@@ -340,6 +347,8 @@ void wxApp::MacReopenApp()
 
         if ( firstIconized )
             firstIconized->Iconize( false ) ;
+        else if ( firstHidden )
+            firstHidden->Show( true );
     }
 }
 
@@ -435,6 +444,7 @@ UInt32 wxIdToMacCommand( int wxId )
 wxMenu* wxFindMenuFromMacCommand( const HICommand &command , wxMenuItem* &item )
 {
     wxMenu* itemMenu = NULL ;
+#ifndef __WXUNIVERSAL__
     int id = 0 ;
 
     // for 'standard' commands which don't have a wx-menu
@@ -477,7 +487,7 @@ wxMenu* wxFindMenuFromMacCommand( const HICommand &command , wxMenuItem* &item )
                 item = (wxMenuItem*) refCon ;
         }
     }
-
+#endif
     return itemMenu ;
 }
 
@@ -513,6 +523,7 @@ wxMacAppMenuEventHandler( EventHandlerCallRef handler , EventRef event , void *d
 {
     wxMacCarbonEvent cEvent( event ) ;
     MenuRef menuRef = cEvent.GetParameter<MenuRef>(kEventParamDirectObject) ;
+#ifndef __WXUNIVERSAL__
     wxMenu* menu = wxFindMenuFromMacMenu( menuRef ) ;
 
     if ( menu )
@@ -555,10 +566,10 @@ wxMacAppMenuEventHandler( EventHandlerCallRef handler , EventRef event , void *d
                 wxWindow *win = menu->GetInvokingWindow();
                 if (win)
                     win->GetEventHandler()->ProcessEvent(wxevent);
-                }
             }
+        }
     }
-
+#endif
     return eventNotHandledErr;
 }
 
@@ -574,72 +585,23 @@ static pascal OSStatus wxMacAppCommandEventHandler( EventHandlerCallRef handler
     wxMenuItem* item = NULL ;
     wxMenu* itemMenu = wxFindMenuFromMacCommand( command , item ) ;
     int id = wxMacCommandToId( command.commandID ) ;
-
+    
     if ( item )
     {
         wxASSERT( itemMenu != NULL ) ;
-
+        
         switch ( cEvent.GetKind() )
         {
             case kEventProcessCommand :
-            {
-                if (item->IsCheckable())
-                    item->Check( !item->IsChecked() ) ;
-
-                if ( itemMenu->SendEvent( id , item->IsCheckable() ? item->IsChecked() : -1 ) )
-                    result = noErr ;
-            }
-            break ;
-
-        case kEventCommandUpdateStatus:
-            {
-                wxUpdateUIEvent event(id);
-                event.SetEventObject( itemMenu );
-
-                bool processed = false;
-
-                // Try the menu's event handler
-                {
-                    wxEvtHandler *handler = itemMenu->GetEventHandler();
-                    if ( handler )
-                        processed = handler->ProcessEvent(event);
-                }
-
-                // Try the window the menu was popped up from
-                // (and up through the hierarchy)
-                if ( !processed )
-                {
-                    const wxMenuBase *menu = itemMenu;
-                    while ( menu )
-                    {
-                        wxWindow *win = menu->GetInvokingWindow();
-                        if ( win )
-                        {
-                            processed = win->GetEventHandler()->ProcessEvent(event);
-                            break;
-                        }
-
-                        menu = menu->GetParent();
-                    }
-                }
-
-                if ( processed )
-                {
-                    // if anything changed, update the changed attribute
-                    if (event.GetSetText())
-                        itemMenu->SetLabel(id, event.GetText());
-                    if (event.GetSetChecked())
-                        itemMenu->Check(id, event.GetChecked());
-                    if (event.GetSetEnabled())
-                        itemMenu->Enable(id, event.GetEnabled());
-
-                    result = noErr ;
-                }
-            }
-            break ;
-
-        default :
-            break ;
+                result = itemMenu->MacHandleCommandProcess( item, id );
+                break ;
+                
+            case kEventCommandUpdateStatus:
+                result = itemMenu->MacHandleCommandUpdateStatus( item, id );
+                break ;
+                
+            default :
+                break ;
         }
     }
     return result ;
@@ -900,6 +862,9 @@ bool wxApp::OnInitGui()
                                sQuitHandler , 0 , FALSE ) ;
     }
 
+    if ( !wxMacInitCocoa() )
+        return false;
+
     return true ;
 }
 
@@ -1101,9 +1066,10 @@ void wxApp::OnIdle(wxIdleEvent& event)
     // either events to the threads other than main or events posted with
     // wxPostEvent() functions
     wxMacProcessNotifierAndPendingEvents();
-
+#ifndef __WXUNIVERSAL__
   if (!wxMenuBar::MacGetInstalledMenuBar() && wxMenuBar::MacGetCommonMenuBar())
     wxMenuBar::MacGetCommonMenuBar()->MacInstallMenuBar();
+#endif
 }
 
 void wxApp::WakeUpIdle()
@@ -1155,6 +1121,15 @@ bool wxApp::Yield(bool onlyIfNeeded)
         return false;
     }
 
+#if wxUSE_THREADS
+    // Yielding from a non-gui thread needs to bail out, otherwise we end up
+    // possibly sending events in the thread too.
+    if ( !wxThread::IsMain() )
+    {
+        return true;
+    }
+#endif // wxUSE_THREADS
+
     s_inYield = true;
 
     // by definition yield should handle all non-processed events
@@ -1194,6 +1169,7 @@ bool wxApp::Yield(bool onlyIfNeeded)
 
 void wxApp::MacDoOneEvent()
 {
+    wxMacAutoreleasePool autoreleasepool;
     EventRef theEvent;
 
     s_inReceiveEvent = true ;
@@ -1626,55 +1602,63 @@ void wxApp::MacCreateKeyEvent( wxKeyEvent& event, wxWindow* focus , long keymess
     {
         // control interferes with some built-in keys like pgdown, return etc. therefore we remove the controlKey modifier
         // and look at the character after
+#ifdef __LP64__
+		// TODO new implementation using TextInputSources
+#else
         UInt32 state = 0;
         UInt32 keyInfo = KeyTranslate((Ptr)GetScriptManagerVariable(smKCHRCache), ( modifiers & (~(controlKey | shiftKey | optionKey))) | keycode, &state);
         keychar = short(keyInfo & charCodeMask);
+#endif
     }
 
     long keyval = wxMacTranslateKey(keychar, keycode) ;
     if ( keyval == keychar && ( event.GetEventType() == wxEVT_KEY_UP || event.GetEventType() == wxEVT_KEY_DOWN ) )
         keyval = wxToupper( keyval ) ;
 
-    // Check for NUMPAD keys
-    if (keyval >= '0' && keyval <= '9' && keycode >= 82 && keycode <= 92)
+    // Check for NUMPAD keys.  For KEY_UP/DOWN events we need to use the
+    // WXK_NUMPAD constants, but for the CHAR event we want to use the
+    // standard ascii values
+    if ( event.GetEventType() != wxEVT_CHAR )
     {
-        keyval = (keyval - '0') + WXK_NUMPAD0;
-    }
-    else if (keycode >= 67 && keycode <= 81)
-    {
-        switch (keycode)
+        if ( keyval >= '0' && keyval <= '9' && keycode >= 82 && keycode <= 92 )
         {
-        case 76 :
-            keyval = WXK_NUMPAD_ENTER;
-            break;
-
-        case 81:
-            keyval = WXK_NUMPAD_EQUAL;
-            break;
-
-        case 67:
-            keyval = WXK_NUMPAD_MULTIPLY;
-            break;
-
-        case 75:
-            keyval = WXK_NUMPAD_DIVIDE;
-            break;
-
-        case 78:
-            keyval = WXK_NUMPAD_SUBTRACT;
-            break;
-
-        case 69:
-            keyval = WXK_NUMPAD_ADD;
-            break;
-
-        case 65:
-            keyval = WXK_NUMPAD_DECIMAL;
-            break;
-
-        default:
-            break;
-        } // end switch
+            keyval = (keyval - '0') + WXK_NUMPAD0;
+        }
+        else if (keycode >= 65 && keycode <= 81) 
+        {
+            switch (keycode)
+            {
+            case 76 :
+                keyval = WXK_NUMPAD_ENTER;
+                break;
+                
+            case 81:
+                keyval = WXK_NUMPAD_EQUAL;
+                break;
+                
+            case 67:
+                keyval = WXK_NUMPAD_MULTIPLY;
+                break;
+                
+            case 75:
+                keyval = WXK_NUMPAD_DIVIDE;
+                break;
+                
+            case 78:
+                keyval = WXK_NUMPAD_SUBTRACT;
+                break;
+                
+            case 69:
+                keyval = WXK_NUMPAD_ADD;
+                break;
+                
+            case 65:
+                keyval = WXK_NUMPAD_DECIMAL;
+                break;
+            default:
+                break;
+            }
+        }
     }
 
     event.m_shiftDown = modifiers & shiftKey;
