@@ -4,7 +4,7 @@
 // Author:      Ryan Norton & Brian Victor
 // Modified by: Royce Mitchell III, Vadim Zeitlin
 // Created:     06/21/02
-// RCS-ID:      $Id: display.cpp,v 1.22 2006/10/02 05:38:00 PC Exp $
+// RCS-ID:      $Id: display.cpp 44636 2007-03-07 19:51:00Z VZ $
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -63,6 +63,7 @@ public:
     }
 
     virtual wxRect GetGeometry() const;
+    virtual wxRect GetClientArea() const;
     virtual wxString GetName() const { return wxString(); }
 
     virtual wxArrayVideoModes GetModes(const wxVideoMode& mode) const;
@@ -173,6 +174,17 @@ wxRect wxDisplayImplMacOSX::GetGeometry() const
                    (int)theRect.size.height ); //floats
 }
 
+wxRect wxDisplayImplMacOSX::GetClientArea() const
+{
+    // VZ: I don't know how to get client area for arbitrary display but
+    //     wxGetClientDisplayRect() does work correctly for at least the main
+    //     one (TODO: do it correctly for the other displays too)
+    if ( IsPrimary() )
+        return wxGetClientDisplayRect();
+
+    return wxDisplayImpl::GetClientArea();
+}
+
 static int wxCFDictKeyToInt( CFDictionaryRef desc, CFStringRef key )
 {
     CFNumberRef value = (CFNumberRef) CFDictionaryGetValue( desc, key );
