@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Agron Selimaj
 // Created:     04/01/98
-// RCS-ID:      $Id: listctrl_mac.cpp,v 1.61.2.2 2007/05/10 02:02:12 VZ Exp $
+// RCS-ID:      $Id: listctrl_mac.cpp 49360 2007-10-23 18:48:40Z SC $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -1076,6 +1076,10 @@ int wxListCtrl::GetCountPerPage() const
 
     if (m_dbImpl)
     {
+        UInt16 height = 1;
+        m_dbImpl->GetDefaultRowHeight( &height );
+        if (height > 0)
+            return GetClientSize().y / height;
     }
 
     return 1;
@@ -1101,7 +1105,15 @@ bool wxListCtrl::GetItem(wxListItem& info) const
         if (!IsVirtual())
         {
             if (info.m_itemId >= 0 && info.m_itemId < GetItemCount())
+            {
                 m_dbImpl->MacGetColumnInfo(info.m_itemId, info.m_col, info);
+                if (info.GetMask() & wxLIST_MASK_STATE)
+                {
+                    DataBrowserItemID id = (DataBrowserItemID)m_dbImpl->GetItemFromLine(info.m_itemId);
+                    if (IsDataBrowserItemSelected( m_dbImpl->GetControlRef(), id ))
+                        info.SetState(info.GetState() | wxLIST_STATE_SELECTED);
+                }
+            }
         }
         else
         {
@@ -2667,7 +2679,9 @@ void wxMacDataBrowserListCtrlControl::DrawItem(
     Rect enclosingRect;
     CGRect enclosingCGRect, iconCGRect, textCGRect;
     Boolean active;
+#ifndef __LP64__
     ThemeDrawingState savedState = NULL;
+#endif
     CGContextRef context = (CGContextRef)list->MacGetDrawingContext();
     RGBColor labelColor;
     labelColor.red = 0;
@@ -2698,9 +2712,10 @@ void wxMacDataBrowserListCtrlControl::DrawItem(
     }
     if (itemState == kDataBrowserItemIsSelected)
     {
-
+#ifndef __LP64__
         GetThemeDrawingState(&savedState);
-        
+#endif
+
         if (active && hasFocus)
         {
             GetThemeBrushAsColor(kThemeBrushAlternatePrimaryHighlightColor, 32, true, &backgroundColor);
@@ -2763,19 +2778,29 @@ void wxMacDataBrowserListCtrlControl::DrawItem(
     }
 
     HIThemeTextHorizontalFlush hFlush = kHIThemeTextHorizontalFlushLeft;
-    UInt16 fontID = kThemeViewsFont;
+    HIThemeTextInfo info;
+
+#ifdef __LP64__
+    info.version = kHIThemeTextInfoVersionOne;
+    info.fontID = kThemeViewsFont;
+    if (font.Ok())
+    {
+        info.fontID = kThemeSpecifiedFont;
+        info.font = (CTFontRef) font.MacGetCTFont();
+    }
+#else
+    info.version = kHIThemeTextInfoVersionZero;
+    info.fontID = kThemeViewsFont;
 
     if (font.Ok())
     {
         if (font.GetFamily() != wxFONTFAMILY_DEFAULT)
-            fontID = font.MacGetThemeFontID();
+            info.fontID = font.MacGetThemeFontID();
 
-// FIXME: replace these with CG or ATSUI calls so we can remove this #ifndef.
-#ifndef __LP64__
         ::TextSize( (short)(font.MacGetFontSize()) ) ;
         ::TextFace( font.MacGetFontStyle() ) ;
-#endif
     }
+#endif
 
     wxListItem item;
     list->GetColumn(listColumn, item);
@@ -2792,10 +2817,7 @@ void wxMacDataBrowserListCtrlControl::DrawItem(
         }
     }
 
-    HIThemeTextInfo info;
-    info.version = kHIThemeTextInfoVersionZero;
     info.state = active ? kThemeStateActive : kThemeStateInactive;
-    info.fontID = fontID;
     info.horizontalFlushness = hFlush;
     info.verticalFlushness = kHIThemeTextVerticalFlushCenter;
     info.options = kHIThemeTextBoxOptionNone;
@@ -2811,8 +2833,10 @@ void wxMacDataBrowserListCtrlControl::DrawItem(
 
     CGContextRestoreGState(context);
 
+#ifndef __LP64__
     if (savedState != NULL)
         SetThemeDrawingState(savedState, true);
+#endif
 }
 
 OSStatus wxMacDataBrowserListCtrlControl::GetSetItemData(DataBrowserItemID itemID,
@@ -2943,7 +2967,6 @@ void wxMacDataBrowserListCtrlControl::ItemNotification(DataBrowserItemID itemID,
         bool trigger = false;
 
         wxListEvent event( wxEVT_COMMAND_LIST_ITEM_SELECTED, list->GetId() );
-        bool isSingle = (list->GetWindowStyle() & wxLC_SINGLE_SEL) != 0;
 
         event.SetEventObject( list );
         if ( !list->IsVirtual() )
@@ -2963,8 +2986,9 @@ void wxMacDataBrowserListCtrlControl::ItemNotification(DataBrowserItemID itemID,
         {
             case kDataBrowserItemDeselected:
                 event.SetEventType(wxEVT_COMMAND_LIST_ITEM_DESELECTED);
-                if ( !isSingle )
-                    trigger = !IsSelectionSuppressed();
+                // as the generic implementation is also triggering this
+                // event for single selection, we do the same (different than listbox)
+                trigger = !IsSelectionSuppressed();
                 break;
 
             case kDataBrowserItemSelected:
@@ -3108,7 +3132,7 @@ void wxMacDataBrowserListCtrlControl::UpdateState(wxMacDataItem* dataItem, wxLis
     bool isSelectedState = (listItem->GetState() == wxLIST_STATE_SELECTED);
 
     // toggle the selection state if wxListInfo state and actual state don't match.
-    if ( isSelected != isSelectedState )
+    if ( listItem->GetMask() & wxLIST_MASK_STATE && isSelected != isSelectedState )
     {
         DataBrowserSetOption options = kDataBrowserItemsAdd;
         if (!isSelectedState)
