@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: menu.cpp,v 1.96.4.1 2007/04/21 23:17:27 KO Exp $
+// RCS-ID:      $Id: menu.cpp 49615 2007-11-04 10:27:33Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -94,14 +94,14 @@ void wxInsertMenuItemsInMenu(wxMenu* menu, MenuRef wm, MenuItemIndex insertAfter
         subMenu = item->GetSubMenu() ;
         if (subMenu)
         {
-            wxInsertMenuItemsInMenu(subMenu, (MenuRef)subMenu->GetHMenu(), 0); 
+            wxInsertMenuItemsInMenu(subMenu, (MenuRef)subMenu->GetHMenu(), 0);
         }
         if ( item->IsSeparator() )
         {
             if ( wm && newItems)
                 InsertMenuItemTextWithCFString( wm,
-                    CFSTR(""), insertAfter, kMenuItemAttrSeparator, 0); 
-        
+                    CFSTR(""), insertAfter, kMenuItemAttrSeparator, 0);
+
             newItems = false;
         }
         else
@@ -110,9 +110,9 @@ void wxInsertMenuItemsInMenu(wxMenu* menu, MenuRef wm, MenuItemIndex insertAfter
                 entry = wxAcceleratorEntry::Create( item->GetText() ) ;
 
             MenuItemIndex winListPos = (MenuItemIndex)-1;
-            OSStatus err = GetIndMenuItemWithCommandID(wm, 
+            OSStatus err = GetIndMenuItemWithCommandID(wm,
                         wxIdToMacCommand ( item->GetId() ), 1, NULL, &winListPos);
-            
+
             if ( wm && err == menuItemNotFoundErr )
             {
                 // NB: the only way to determine whether or not we should add
@@ -126,7 +126,7 @@ void wxInsertMenuItemsInMenu(wxMenu* menu, MenuRef wm, MenuItemIndex insertAfter
 
             delete entry ;
         }
-    }  
+    }
 }
 
 // ============================================================================
@@ -210,10 +210,10 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *pItem, size_t pos)
     {
         if ( pos == (size_t)-1 )
             AppendMenuItemTextWithCFString( MAC_WXHMENU(m_hMenu),
-                CFSTR(""), kMenuItemAttrSeparator, 0,NULL); 
+                CFSTR(""), kMenuItemAttrSeparator, 0,NULL);
         else
             InsertMenuItemTextWithCFString( MAC_WXHMENU(m_hMenu),
-                CFSTR(""), pos, kMenuItemAttrSeparator, 0); 
+                CFSTR(""), pos, kMenuItemAttrSeparator, 0);
     }
     else
     {
@@ -554,6 +554,82 @@ void wxMenu::MacAfterDisplay( bool isSubMenu )
     }
 }
 
+wxInt32 wxMenu::MacHandleCommandProcess( wxMenuItem* item, int id, wxWindow* targetWindow )
+{
+    OSStatus result = eventNotHandledErr ;
+    if (item->IsCheckable())
+        item->Check( !item->IsChecked() ) ;
+
+    if ( SendEvent( id , item->IsCheckable() ? item->IsChecked() : -1 ) )
+        result = noErr ;
+    else
+    {
+        if ( targetWindow != NULL )
+        {
+            wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED , id);
+            event.SetEventObject(targetWindow);
+            event.SetInt(item->IsCheckable() ? item->IsChecked() : -1);
+
+            if ( targetWindow->GetEventHandler()->ProcessEvent(event) )
+                result = noErr ;
+        }
+    }
+    return result;
+}
+
+wxInt32 wxMenu::MacHandleCommandUpdateStatus( wxMenuItem* item, int id, wxWindow* targetWindow )
+{
+    OSStatus result = eventNotHandledErr ;
+    wxUpdateUIEvent event(id);
+    event.SetEventObject( this );
+
+    bool processed = false;
+
+    // Try the menu's event handler
+    {
+        wxEvtHandler *handler = GetEventHandler();
+        if ( handler )
+            processed = handler->ProcessEvent(event);
+    }
+
+    // Try the window the menu was popped up from
+    // (and up through the hierarchy)
+    if ( !processed )
+    {
+        const wxMenuBase *menu = this;
+        while ( menu )
+        {
+            wxWindow *win = menu->GetInvokingWindow();
+            if ( win )
+            {
+                processed = win->GetEventHandler()->ProcessEvent(event);
+                break;
+            }
+
+            menu = menu->GetParent();
+        }
+    }
+
+    if ( !processed && targetWindow != NULL)
+    {
+        processed = targetWindow->GetEventHandler()->ProcessEvent(event);
+    }
+    
+    if ( processed )
+    {
+        // if anything changed, update the changed attribute
+        if (event.GetSetText())
+            SetLabel(id, event.GetText());
+        if (event.GetSetChecked())
+            Check(id, event.GetChecked());
+        if (event.GetSetEnabled())
+            Enable(id, event.GetEnabled());
+
+        result = noErr ;
+    }
+    return result;
+}
+
 // Menu Bar
 
 /*
@@ -655,10 +731,10 @@ void wxMenuBar::MacInstallMenuBar()
     // However, the change from 10.2 to 10.3 suggests it is preferred
 #if TARGET_API_MAC_OSX
     InsertMenuItemTextWithCFString( appleMenu,
-                CFSTR(""), 0, kMenuItemAttrSeparator, 0); 
+                CFSTR(""), 0, kMenuItemAttrSeparator, 0);
 #endif
     InsertMenuItemTextWithCFString( appleMenu,
-                CFSTR("About..."), 0, 0, 0); 
+                CFSTR("About..."), 0, 0, 0);
     MacInsertMenu( appleMenu , 0 ) ;
 
     // clean-up the help menu before adding new items
@@ -738,7 +814,7 @@ void wxMenuBar::MacInstallMenuBar()
                     {
                         if ( mh )
                             AppendMenuItemTextWithCFString( mh,
-                                CFSTR(""), kMenuItemAttrSeparator, 0,NULL); 
+                                CFSTR(""), kMenuItemAttrSeparator, 0,NULL);
                     }
                     else
                     {
@@ -754,8 +830,10 @@ void wxMenuBar::MacInstallMenuBar()
                             if ( mh )
                             {
                                 UMAAppendMenuItem(mh, wxStripMenuCodes(item->GetText()) , wxFont::GetDefaultEncoding(), entry);
-                                SetMenuItemCommandID( mh , CountMenuItems(mh) , wxIdToMacCommand ( item->GetId() ) ) ;
-                                SetMenuItemRefCon( mh , CountMenuItems(mh) , (URefCon) item ) ;
+                                MenuItemIndex position = CountMenuItems(mh);
+                                SetMenuItemCommandID( mh , position, wxIdToMacCommand ( item->GetId() ) );
+                                SetMenuItemRefCon( mh , position, (URefCon) item );
+                                item->DoUpdateItemBitmap( mh, position );
                             }
                         }
 
@@ -764,42 +842,42 @@ void wxMenuBar::MacInstallMenuBar()
                 }
             }
         }
-        
-        else if ( ( m_titles[i] == wxT("Window") || m_titles[i] == wxT("&Window") ) 
+
+        else if ( ( m_titles[i] == wxT("Window") || m_titles[i] == wxT("&Window") )
                 && GetAutoWindowMenu() )
-        {           
+        {
             if ( MacGetWindowMenuHMenu() == NULL )
             {
                 CreateStandardWindowMenu( 0 , (MenuHandle*) &s_macWindowMenuHandle ) ;
             }
-            
+
             MenuRef wm = (MenuRef)MacGetWindowMenuHMenu();
             if ( wm == NULL )
                 break;
-            
+
             // get the insertion point in the standard menu
             MenuItemIndex winListStart;
-            GetIndMenuItemWithCommandID(wm, 
+            GetIndMenuItemWithCommandID(wm,
                         kHICommandWindowListSeparator, 1, NULL, &winListStart);
-            
+
             // add a separator so that the standard items and the custom items
             // aren't mixed together, but only if this is the first run
-            OSStatus err = GetIndMenuItemWithCommandID(wm, 
+            OSStatus err = GetIndMenuItemWithCommandID(wm,
                         'WXWM', 1, NULL, NULL);
-            
+
             if ( err == menuItemNotFoundErr )
             {
                 InsertMenuItemTextWithCFString( wm,
                         CFSTR(""), winListStart-1, kMenuItemAttrSeparator, 'WXWM');
             }
-            
-            wxInsertMenuItemsInMenu(menu, wm, winListStart);    
+
+            wxInsertMenuItemsInMenu(menu, wm, winListStart);
         }
         else
         {
             UMASetMenuTitle( MAC_WXHMENU(menu->GetHMenu()) , m_titles[i], m_font.GetEncoding()  ) ;
             menu->MacBeforeDisplay(false) ;
-            
+
             ::InsertMenu(MAC_WXHMENU(_wxMenuAt(m_menus, i)->GetHMenu()), 0);
         }
     }
@@ -874,6 +952,15 @@ wxString wxMenuBar::GetLabelTop(size_t pos) const
     wxCHECK_MSG( pos < GetMenuCount(), wxEmptyString,
                  wxT("invalid menu index in wxMenuBar::GetLabelTop") );
 
+    return wxStripMenuCodes(m_titles[pos]);
+}
+
+// Gets the original label at the top-level of the menubar
+wxString wxMenuBar::GetMenuLabel(size_t pos) const
+{
+    wxCHECK_MSG( pos < GetMenuCount(), wxEmptyString,
+                 wxT("invalid menu index in wxMenuBar::GetMenuLabel") );
+
     return m_titles[pos];
 }
 
