@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin to derive from wxChoiceBase
 // Created:     04/01/98
-// RCS-ID:      $Id: choice.cpp,v 1.122 2006/10/31 08:49:41 RD Exp $
+// RCS-ID:      $Id: choice.cpp 45011 2007-03-22 02:46:21Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -720,6 +720,13 @@ bool wxChoice::MSWCommand(WXUINT param, WXWORD WXUNUSED(id))
             // need to reset the selection back to it if it's eventually
             // cancelled by user
             m_lastAcceptedSelection = GetCurrentSelection();
+            if ( m_lastAcceptedSelection == -1 )
+            {
+                // no current selection so no need to restore it later (this
+                // happens when opening a combobox containing text not from its
+                // list of items and we shouldn't erase this text)
+                m_lastAcceptedSelection = wxID_NONE;
+            }
             break;
 
         case CBN_CLOSEUP:
