@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Agron Selimaj
 // Created:     04/01/98
-// RCS-ID:      $Id: listctrl.cpp,v 1.272 2007/01/01 20:48:51 RD Exp $
+// RCS-ID:      $Id: listctrl.cpp 46383 2007-06-09 10:58:38Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -205,7 +205,7 @@ public:
    {
        if (attr)
            delete attr;
-   };
+   }
 
     DECLARE_NO_COPY_CLASS(wxListItemInternalData)
 };
@@ -948,7 +948,7 @@ wxUIntPtr wxListCtrl::GetItemData(long item) const
 }
 
 // Sets the item data
-bool wxListCtrl::SetItemData(long item, long data)
+bool wxListCtrl::SetItemPtrData(long item, wxUIntPtr data)
 {
     wxListItem info;
 
@@ -959,6 +959,11 @@ bool wxListCtrl::SetItemData(long item, long data)
     return SetItem(info);
 }
 
+bool wxListCtrl::SetItemData(long item, long data)
+{
+    return SetItemPtrData(item, data);
+}
+
 wxRect wxListCtrl::GetViewRect() const
 {
     wxASSERT_MSG( !HasFlag(wxLC_REPORT | wxLC_LIST),
@@ -2418,22 +2423,23 @@ static RECT GetCustomDrawnItemRect(const NMCUSTOMDRAW& nmcd)
     return rc;
 }
 
-static void HandleSubItemPrepaint(LPNMLVCUSTOMDRAW pLVCD, HFONT hfont)
+static
+bool HandleSubItemPrepaint(LPNMLVCUSTOMDRAW pLVCD, HFONT hfont, int colCount)
 {
     NMCUSTOMDRAW& nmcd = pLVCD->nmcd;
 
     HDC hdc = nmcd.hdc;
     HWND hwndList = nmcd.hdr.hwndFrom;
+    const int col = pLVCD->iSubItem;
     const DWORD item = nmcd.dwItemSpec;
 
-
     // the font must be valid, otherwise we wouldn't be painting the item at all
     SelectInHDC selFont(hdc, hfont);
 
     // get the rectangle to paint
     RECT rc;
-    ListView_GetSubItemRect(hwndList, item, pLVCD->iSubItem, LVIR_BOUNDS, &rc);
-    if ( !pLVCD->iSubItem )
+    ListView_GetSubItemRect(hwndList, item, col, LVIR_BOUNDS, &rc);
+    if ( !col && colCount > 1 )
     {
         // broken ListView_GetSubItemRect() returns the entire item rect for
         // 0th subitem while we really need just the part for this column
@@ -2454,7 +2460,7 @@ static void HandleSubItemPrepaint(LPNMLVCUSTOMDRAW pLVCD, HFONT hfont)
     wxZeroMemory(it);
     it.mask = LVIF_TEXT | LVIF_IMAGE;
     it.iItem = item;
-    it.iSubItem = pLVCD->iSubItem;
+    it.iSubItem = col;
     it.pszText = text;
     it.cchTextMax = WXSIZEOF(text);
     ListView_GetItem(hwndList, &it);
@@ -2484,12 +2490,38 @@ static void HandleSubItemPrepaint(LPNMLVCUSTOMDRAW pLVCD, HFONT hfont)
 
     ::SetBkMode(hdc, TRANSPARENT);
 
-    // TODO: support for centred/right aligned columns
-    ::DrawText(hdc, text, -1, &rc,
+    UINT fmt = DT_SINGLELINE |
 #ifndef __WXWINCE__
                DT_WORD_ELLIPSIS |
 #endif // __WXWINCE__
-               DT_NOPREFIX | DT_SINGLELINE | DT_VCENTER);
+               DT_NOPREFIX |
+               DT_VCENTER;
+
+    LV_COLUMN lvCol;
+    wxZeroMemory(lvCol);
+    lvCol.mask = LVCF_FMT;
+    if ( ListView_GetColumn(hwndList, col, &lvCol) )
+    {
+        switch ( lvCol.fmt & LVCFMT_JUSTIFYMASK )
+        {
+            case LVCFMT_LEFT:
+                fmt |= DT_LEFT;
+                break;
+
+            case LVCFMT_CENTER:
+                fmt |= DT_CENTER;
+                break;
+
+            case LVCFMT_RIGHT:
+                fmt |= DT_RIGHT;
+                break;
+        }
+    }
+    //else: failed to get alignment, assume it's DT_LEFT (default)
+
+    DrawText(hdc, text, -1, &rc, fmt);
+
+    return true;
 }
 
 static void HandleItemPostpaint(NMCUSTOMDRAW nmcd)
@@ -2577,7 +2609,7 @@ static void HandleItemPaint(LPNMLVCUSTOMDRAW pLVCD, HFONT hfont)
     for ( int col = 0; col < colCount; col++ )
     {
         pLVCD->iSubItem = col;
-        HandleSubItemPrepaint(pLVCD, hfont);
+        HandleSubItemPrepaint(pLVCD, hfont, colCount);
     }
 
     HandleItemPostpaint(nmcd);
