@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin
 // Created:     04/01/98
-// RCS-ID:      $Id: menu.cpp,v 1.143 2007/01/11 01:21:42 VZ Exp $
+// RCS-ID:      $Id: menu.cpp 48053 2007-08-13 17:07:01Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -437,10 +437,9 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *pItem, size_t pos)
             // while our owner-drawn code is not
 
             // first compile-time check
+            // MIIM_BITMAP only works under Win98/2000+
 #if defined(MIIM_BITMAP) && (_WIN32_WINNT >= 0x0500)
-            WinStruct<MENUITEMINFO> mii;
-
-            // now run-time one: MIIM_BITMAP only works under WinME/2000+
+            WinStruct<wxMENUITEMINFO_> mii;
             if ( wxGetWinVersion() >= wxWinVersion_98 )
             {
                 mii.fMask = MIIM_STRING | MIIM_DATA | MIIM_BITMAP;
@@ -480,7 +479,8 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *pItem, size_t pos)
                 }
                 mii.hbmpItem = HBMMENU_CALLBACK;
 
-                ok = ::InsertMenuItem(GetHmenu(), pos, TRUE /* by pos */, &mii);
+                ok = ::InsertMenuItem(GetHmenu(), pos, TRUE /* by pos */,
+                                      (MENUITEMINFO*)&mii);
                 if ( !ok )
                 {
                     wxLogLastError(wxT("InsertMenuItem()"));
@@ -512,7 +512,7 @@ bool wxMenu::DoInsertOrAppend(wxMenuItem *pItem, size_t pos)
                     pItem->ResetOwnerDrawn();
                 }
             }
-#endif // MIIM_BITMAP
+#endif // defined(MIIM_BITMAP) && (_WIN32_WINNT >= 0x0500)
         }
 
         if ( !ok )
@@ -777,10 +777,15 @@ bool wxMenu::MSWCommand(WXUINT WXUNUSED(param), WXWORD id)
     // ignore commands from the menu title
     if ( id != (WXWORD)idMenuTitle )
     {
-        // get the checked status of the command: notice that menuState is the
-        // old state of the menu, so the test for MF_CHECKED must be inverted
+        // update the check item when it's clicked
+        wxMenuItem * const item = FindItem(id);
+        if ( item && item->IsCheckable() )
+            item->Toggle();
+
+        // get the status of the menu item: note that it has been just changed
+        // by Toggle() above so here we already get the new state of the item
         UINT menuState = ::GetMenuState(GetHmenu(), id, MF_BYCOMMAND);
-        SendEvent(id, !(menuState & MF_CHECKED));
+        SendEvent(id, menuState & MF_CHECKED);
     }
 
     return true;
@@ -1081,6 +1086,15 @@ wxString wxMenuBar::GetLabelTop(size_t pos) const
     return wxMenuItem::GetLabelFromText(m_titles[pos]);
 }
 
+// Gets the original label at the top-level of the menubar
+wxString wxMenuBar::GetMenuLabel(size_t pos) const
+{
+    wxCHECK_MSG( pos < GetMenuCount(), wxEmptyString,
+                 wxT("invalid menu index in wxMenuBar::GetMenuLabel") );
+
+    return m_titles[pos];
+}
+
 // ---------------------------------------------------------------------------
 // wxMenuBar construction
 // ---------------------------------------------------------------------------
