@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     11/6/98
-// RCS-ID:      $Id: automtn.cpp,v 1.52 2006/11/12 16:30:38 JS Exp $
+// RCS-ID:      $Id: automtn.cpp 44961 2007-03-19 20:00:59Z VZ $
 // Copyright:   (c) 1998, Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -688,7 +688,7 @@ WXDLLEXPORT bool wxConvertOleToVariant(const VARIANTARG& oleVariant, wxVariant&
             unsigned short dosTime = 0;
             VariantTimeToDosDateTime(oleVariant.date, & dosDate, & dosTime);
 
-            long dosDateTime = (dosDate << 16) || dosTime;
+            long dosDateTime = (dosDate << 16) | dosTime;
             wxDateTime date;
             date.SetFromDOS(dosDateTime);
             variant = date;
