@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: tbar95.cpp,v 1.192.2.2 2007/03/27 13:41:12 JS Exp $
+// RCS-ID:      $Id: tbar95.cpp 48849 2007-09-21 10:33:26Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -265,6 +265,13 @@ bool wxToolBar::MSWCreateToolbar(const wxPoint& pos, const wxSize& size)
     // toolbar-specific post initialisation
     ::SendMessage(GetHwnd(), TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0);
 
+    // Fix a bug on e.g. the Silver theme on WinXP where control backgrounds
+    // are incorrectly drawn, by forcing the background to a specific colour.
+    int majorVersion, minorVersion;
+    wxGetOsVersion(& majorVersion, & minorVersion);
+    if (majorVersion < 6)
+        SetBackgroundColour(GetBackgroundColour());
+
     return true;
 }
 
@@ -362,8 +369,16 @@ wxSize wxToolBar::DoGetBestSize() const
         sizeBest.y = size.cy;
     }
 
-    if (!IsVertical() && !(GetWindowStyle() & wxTB_NODIVIDER))
-        sizeBest.y += 1;
+    if (!IsVertical())
+    {
+        // Without the extra height, DoGetBestSize can report a size that's
+        // smaller than the actual window, causing windows to overlap slightly
+        // in some circumstances, leading to missing borders (especially noticeable
+        // in AUI layouts).
+        if (!(GetWindowStyle() & wxTB_NODIVIDER))
+            sizeBest.y += 2;
+        sizeBest.y ++;
+    }
 
     CacheBestSize(sizeBest);
 
@@ -886,7 +901,7 @@ bool wxToolBar::Realize()
                                 DoToggleTool(tool, true);
                             }
                         }
-                        else if (tool->IsToggled())
+                        else if ( tool->IsToggled() )
                         {
                             wxToolBarToolsList::compatibility_iterator nodePrev = node->GetPrevious();
                             int prevIndex = i - 1;
@@ -900,7 +915,7 @@ bool wxToolBar::Realize()
                                 if ( tool->Toggle(false) )
                                     DoToggleTool(tool, false);
 
-                                prevButton.fsState = TBSTATE_ENABLED;
+                                prevButton.fsState &= ~TBSTATE_CHECKED;
                                 nodePrev = nodePrev->GetPrevious();
                                 prevIndex--;
                             }
@@ -1412,6 +1427,9 @@ void wxToolBar::OnEraseBackground(wxEraseEvent& event)
     RECT rect = wxGetClientRect(GetHwnd());
     HDC hdc = GetHdcOf((*event.GetDC()));
 
+    int majorVersion, minorVersion;
+    wxGetOsVersion(& majorVersion, & minorVersion);
+
 #if wxUSE_UXTHEME
     // we may need to draw themed colour so that we appear correctly on
     // e.g. notebook page under XP with themes but only do it if the parent
@@ -1432,6 +1450,30 @@ void wxToolBar::OnEraseBackground(wxEraseEvent& event)
                 wxLogApiError(_T("DrawThemeParentBackground(toolbar)"), hr);
         }
     }
+
+    // Only draw a rebar theme on Vista, since it doesn't jive so well with XP
+    if ( !UseBgCol() && majorVersion >= 6 )
+    {
+        wxUxThemeEngine *theme = wxUxThemeEngine::GetIfActive();
+        if ( theme )
+        {
+            wxUxThemeHandle hTheme(this, L"REBAR");
+
+            RECT r;
+            wxRect rect = GetClientRect();
+            wxCopyRectToRECT(rect, r);
+
+            HRESULT hr = theme->DrawThemeBackground(hTheme, hdc, 0, 0, & r, NULL);
+            if ( hr == S_OK )
+                return;
+
+            // it can also return S_FALSE which seems to simply say that it
+            // didn't draw anything but no error really occurred
+            if ( FAILED(hr) )
+                wxLogApiError(_T("DrawThemeBackground(toolbar)"), hr);
+        }
+    }
+
 #endif // wxUSE_UXTHEME
 
     if ( UseBgCol() || (GetMSWToolbarStyle() & TBSTYLE_TRANSPARENT) )
@@ -1516,6 +1558,11 @@ bool wxToolBar::HandlePaint(WXWPARAM wParam, WXLPARAM lParam)
         // no controls, nothing to erase
         return false;
 
+    wxSize clientSize = GetClientSize();
+    int majorVersion, minorVersion;
+    wxGetOsVersion(& majorVersion, & minorVersion);
+
+    // prepare the DC on which we'll be drawing
     // prepare the DC on which we'll be drawing
     wxClientDC dc(this);
     dc.SetBrush(wxBrush(GetBackgroundColour(), wxSOLID));
@@ -1582,7 +1629,39 @@ bool wxToolBar::HandlePaint(WXWPARAM wParam, WXLPARAM lParam)
                 if ( rectCtrl.Intersects(rectItem) )
                 {
                     // yes, do erase it!
-                    dc.DrawRectangle(rectItem);
+
+                    bool haveRefreshed = false;
+
+#if wxUSE_UXTHEME
+                    if ( !UseBgCol() && !GetParent()->UseBgCol() )
+                    {
+                        // Don't use DrawThemeBackground
+                    }
+                    else if (!UseBgCol() && majorVersion >= 6 )
+                    {
+                        wxUxThemeEngine *theme = wxUxThemeEngine::GetIfActive();
+                        if ( theme )
+                        {
+                            wxUxThemeHandle hTheme(this, L"REBAR");
+
+                            RECT clipRect = r;
+
+                            // Draw the whole background since the pattern may be position sensitive;
+                            // but clip it to the area of interest.
+                            r.left = 0;
+                            r.right = clientSize.x;
+                            r.top = 0;
+                            r.bottom = clientSize.y;
+
+                            HRESULT hr = theme->DrawThemeBackground(hTheme, (HDC) dc.GetHDC(), 0, 0, & r, & clipRect);
+                            if ( hr == S_OK )
+                                haveRefreshed = true;
+                        }
+                    }
+#endif
+
+                    if (!haveRefreshed)
+                        dc.DrawRectangle(rectItem);
 
                     // Necessary in case we use a no-paint-on-size
                     // style in the parent: the controls can disappear
