@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     2005-09-30
-// RCS-ID:      $Id: richtextctrl.cpp,v 1.62 2006/12/29 19:37:04 JS Exp $
+// RCS-ID:      $Id: richtextctrl.cpp 49952 2007-11-14 14:53:36Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -129,6 +129,13 @@ wxRichTextCtrl::wxRichTextCtrl(wxWindow* parent,
 bool wxRichTextCtrl::Create( wxWindow* parent, wxWindowID id, const wxString& value, const wxPoint& pos, const wxSize& size, long style,
                              const wxValidator& validator, const wxString& name)
 {
+    if ((style & wxBORDER_MASK) == wxBORDER_DEFAULT)
+#ifdef __WXMSW__
+        style |= GetThemedBorderStyle();
+#else
+        style |= wxBORDER_SUNKEN;
+#endif
+
     if (!wxTextCtrlBase::Create(parent, id, pos, size,
                                 style|wxFULL_REPAINT_ON_RESIZE,
                                 validator, name))
@@ -141,7 +148,7 @@ bool wxRichTextCtrl::Create( wxWindow* parent, wxWindowID id, const wxString& va
 
     GetBuffer().Reset();
     GetBuffer().SetRichTextCtrl(this);
-    
+
     SetCaret(new wxCaret(this, wxRICHTEXT_DEFAULT_CARET_WIDTH, 16));
     GetCaret()->Show();
 
@@ -151,8 +158,7 @@ bool wxRichTextCtrl::Create( wxWindow* parent, wxWindowID id, const wxString& va
     // The base attributes must all have default values
     wxTextAttrEx attributes;
     attributes.SetFont(GetFont());
-    attributes.SetTextColour(*wxBLACK);
-    attributes.SetBackgroundColour(*wxWHITE);
+    attributes.SetTextColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
     attributes.SetAlignment(wxTEXT_ALIGNMENT_LEFT);
     attributes.SetLineSpacing(10);
     attributes.SetParagraphSpacingAfter(10);
@@ -167,7 +173,7 @@ bool wxRichTextCtrl::Create( wxWindow* parent, wxWindowID id, const wxString& va
     wxTextAttrEx defaultAttributes;
     SetDefaultStyle(defaultAttributes);
 
-    SetBackgroundColour(*wxWHITE);
+    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
     SetBackgroundStyle(wxBG_STYLE_CUSTOM);
 
     // Tell the sizers to use the given or best size
@@ -194,7 +200,7 @@ bool wxRichTextCtrl::Create( wxWindow* parent, wxWindowID id, const wxString& va
 wxRichTextCtrl::~wxRichTextCtrl()
 {
     GetBuffer().RemoveEventHandler(this);
-    
+
     delete m_contextMenu;
 }
 
@@ -230,7 +236,11 @@ void wxRichTextCtrl::Thaw()
 
     if (m_freezeCount == 0)
     {
-        SetupScrollbars();
+        if (GetBuffer().GetDirty())
+            LayoutContent();
+        else
+            SetupScrollbars();
+
         Refresh(false);
     }
 }
@@ -245,11 +255,9 @@ void wxRichTextCtrl::Clear()
     m_caretAtLineStart = false;
     m_selectionRange.SetRange(-2, -2);
 
-    SetScrollbars(0, 0, 0, 0, 0, 0);
-
     if (m_freezeCount == 0)
     {
-        SetupScrollbars();
+        LayoutContent();
         Refresh(false);
     }
     SendTextUpdatedEvent();
@@ -328,7 +336,7 @@ void wxRichTextCtrl::OnKillFocus(wxFocusEvent& WXUNUSED(event))
 
 void wxRichTextCtrl::OnCaptureLost(wxMouseCaptureLostEvent& WXUNUSED(event))
 {
-    m_dragging = false;    
+    m_dragging = false;
 }
 
 /// Left-click
@@ -390,7 +398,7 @@ void wxRichTextCtrl::OnLeftUp(wxMouseEvent& event)
         long position = 0;
         wxPoint logicalPt = event.GetLogicalPosition(dc);
         int hit = GetBuffer().HitTest(dc, logicalPt, position);
-    
+
         if (hit != wxRICHTEXT_HITTEST_NONE)
         {
             wxTextAttrEx attr;
@@ -402,20 +410,20 @@ void wxRichTextCtrl::OnLeftUp(wxMouseEvent& event)
                     if (!urlTarget.IsEmpty())
                     {
                         wxMouseEvent mouseEvent(event);
-                        
+
                         long startPos = 0, endPos = 0;
                         wxRichTextObject* obj = GetBuffer().GetLeafObjectAtPosition(position);
                         if (obj)
                         {
                             startPos = obj->GetRange().GetStart();
                             endPos = obj->GetRange().GetEnd();
-                        }                        
-                        
+                        }
+
                         wxTextUrlEvent urlEvent(GetId(), mouseEvent, startPos, endPos);
                         InitCommandEvent(urlEvent);
 
                         urlEvent.SetString(urlTarget);
-                        
+
                         GetEventHandler()->ProcessEvent(urlEvent);
                     }
                 }
@@ -434,11 +442,11 @@ void wxRichTextCtrl::OnMoveMouse(wxMouseEvent& event)
     long position = 0;
     wxPoint logicalPt = event.GetLogicalPosition(dc);
     int hit = GetBuffer().HitTest(dc, logicalPt, position);
-    
+
     // See if we need to change the cursor
-    
+
     {
-        if (hit != wxRICHTEXT_HITTEST_NONE)
+        if (hit != wxRICHTEXT_HITTEST_NONE && !(hit & wxRICHTEXT_HITTEST_OUTSIDE))
         {
             wxTextAttrEx attr;
             if (GetStyle(position, attr))
@@ -453,6 +461,8 @@ void wxRichTextCtrl::OnMoveMouse(wxMouseEvent& event)
                 }
             }
         }
+        else
+            SetCursor(m_textCursor);
     }
 
     if (!event.Dragging())
@@ -588,7 +598,7 @@ void wxRichTextCtrl::OnChar(wxKeyEvent& event)
             // Generate conventional event
             wxCommandEvent textEvent(wxEVT_COMMAND_TEXT_ENTER, GetId());
             InitCommandEvent(textEvent);
-            
+
             GetEventHandler()->ProcessEvent(textEvent);
         }
         Update();
@@ -596,7 +606,7 @@ void wxRichTextCtrl::OnChar(wxKeyEvent& event)
     else if (event.GetKeyCode() == WXK_BACK)
     {
         BeginBatchUndo(_("Delete Text"));
-        
+
         // Submit range in character positions, which are greater than caret positions,
         // so subtract 1 for deleted character and add 1 for conversion to character position.
         if (m_caretPosition > -1 && !HasSelection())
@@ -668,7 +678,6 @@ void wxRichTextCtrl::OnChar(wxKeyEvent& event)
         switch ( keycode )
         {
             case WXK_ESCAPE:
-            // case WXK_SPACE:
             case WXK_DELETE:
             case WXK_START:
             case WXK_LBUTTON:
@@ -762,6 +771,7 @@ void wxRichTextCtrl::OnChar(wxKeyEvent& event)
             case WXK_NUMPAD_SEPARATOR:
             case WXK_NUMPAD_SUBTRACT:
             case WXK_NUMPAD_DECIMAL:
+            case WXK_WINDOWS_LEFT:
             {
                 event.Skip();
                 return;
@@ -780,9 +790,13 @@ void wxRichTextCtrl::OnChar(wxKeyEvent& event)
                     GetId());
                 cmdEvent.SetEventObject(this);
                 cmdEvent.SetFlags(flags);
+#if wxUSE_UNICODE
+                cmdEvent.SetCharacter(event.GetUnicodeKey());
+#else
                 cmdEvent.SetCharacter((wxChar) keycode);
+#endif
                 cmdEvent.SetPosition(m_caretPosition+1);
-                
+
                 if (keycode == wxT('\t'))
                 {
                     // See if we need to promote or demote the selection or paragraph at the cursor
@@ -812,14 +826,18 @@ void wxRichTextCtrl::OnChar(wxKeyEvent& event)
                 long newPos = m_caretPosition;
                 DeleteSelectedContent(& newPos);
 
+#if wxUSE_UNICODE
+                wxString str = event.GetUnicodeKey();
+#else
                 wxString str = (wxChar) event.GetKeyCode();
+#endif
                 GetBuffer().InsertTextWithUndo(newPos+1, str, this, 0);
 
                 EndBatchUndo();
 
                 SetDefaultStyleToCursorStyle();
                 ScrollIntoView(m_caretPosition, WXK_RIGHT);
-                
+
                 GetEventHandler()->ProcessEvent(cmdEvent);
 
                 Update();
@@ -1007,7 +1025,7 @@ bool wxRichTextCtrl::ScrollIntoView(long position, int keyCode)
 
     // Going down
     if (keyCode == WXK_DOWN || keyCode == WXK_NUMPAD_DOWN ||
-        keyCode == WXK_RIGHT || keyCode == WXK_NUMPAD_DOWN ||
+        keyCode == WXK_RIGHT || keyCode == WXK_NUMPAD_RIGHT ||
         keyCode == WXK_END || keyCode == WXK_NUMPAD_END ||
         keyCode == WXK_PAGEDOWN || keyCode == WXK_NUMPAD_PAGEDOWN)
     {
@@ -1100,17 +1118,10 @@ bool wxRichTextCtrl::IsPositionVisible(long pos) const
     startX = 0;
     startY = startY * ppuY;
 
-    int sx = 0, sy = 0;
-    GetVirtualSize(& sx, & sy);
-    sx = 0;
-    if (ppuY != 0)
-        sy = sy/ppuY;
-
     wxRect rect = line->GetRect();
-
     wxSize clientSize = GetClientSize();
 
-    return !(((rect.y + rect.height) > (clientSize.y + startY)) || rect.y < startY);
+    return (rect.GetBottom() > startY) && (rect.GetTop() < (startY + clientSize.y));
 }
 
 void wxRichTextCtrl::SetCaretPosition(long position, bool showAtLineStart)
@@ -1331,7 +1342,7 @@ bool wxRichTextCtrl::MoveDown(int noLines, int flags)
         // we want to be at the end of the last line but with m_caretAtLineStart set to true,
         // so we view the caret at the start of the line.
         bool caretLineStart = false;
-        if (hitTest == wxRICHTEXT_HITTEST_BEFORE)
+        if (hitTest & wxRICHTEXT_HITTEST_BEFORE)
         {
             wxRichTextLine* thisLine = GetBuffer().GetLineAtPosition(newPos-1);
             wxRichTextRange lineRange;
@@ -1554,6 +1565,11 @@ bool wxRichTextCtrl::PageDown(int noPages, int flags)
     return false;
 }
 
+static bool wxRichTextCtrlIsWhitespace(const wxString& str)
+{
+    return str == wxT(" ") || str == wxT("\t");
+}
+
 // Finds the caret position for the next word
 long wxRichTextCtrl::FindNextWordPosition(int direction) const
 {
@@ -1568,7 +1584,12 @@ long wxRichTextCtrl::FindNextWordPosition(int direction) const
         {
             // i is in character, not caret positions
             wxString text = GetBuffer().GetTextForRange(wxRichTextRange(i, i));
-            if (text != wxT(" ") && !text.empty())
+            wxRichTextLine* line = GetBuffer().GetLineAtPosition(i, false);
+            if (line && (i == line->GetAbsoluteRange().GetEnd()))
+            {
+                break;
+            }
+            else if (!wxRichTextCtrlIsWhitespace(text) && !text.empty())
                 i += direction;
             else
             {
@@ -1579,9 +1600,13 @@ long wxRichTextCtrl::FindNextWordPosition(int direction) const
         {
             // i is in character, not caret positions
             wxString text = GetBuffer().GetTextForRange(wxRichTextRange(i, i));
+            wxRichTextLine* line = GetBuffer().GetLineAtPosition(i, false);
+            if (line && (i == line->GetAbsoluteRange().GetEnd()))
+                return wxMax(-1, i);
+
             if (text.empty()) // End of paragraph, or maybe an image
                 return wxMax(-1, i - 1);
-            else if (text == wxT(" ") || text.empty())
+            else if (wxRichTextCtrlIsWhitespace(text) || text.empty())
                 i += direction;
             else
             {
@@ -1602,9 +1627,11 @@ long wxRichTextCtrl::FindNextWordPosition(int direction) const
         {
             // i is in character, not caret positions
             wxString text = GetBuffer().GetTextForRange(wxRichTextRange(i, i));
-            if (text.empty()) // End of paragraph, or maybe an image
+            wxRichTextLine* line = GetBuffer().GetLineAtPosition(i, false);
+
+            if (text.empty() || (line && (i == line->GetAbsoluteRange().GetStart()))) // End of paragraph, or maybe an image
                 break;
-            else if (text == wxT(" ") || text.empty())
+            else if (wxRichTextCtrlIsWhitespace(text) || text.empty())
                 i += direction;
             else
                 break;
@@ -1614,7 +1641,11 @@ long wxRichTextCtrl::FindNextWordPosition(int direction) const
         {
             // i is in character, not caret positions
             wxString text = GetBuffer().GetTextForRange(wxRichTextRange(i, i));
-            if (text != wxT(" ") /* && !text.empty() */)
+            wxRichTextLine* line = GetBuffer().GetLineAtPosition(i, false);
+            if (line && line->GetAbsoluteRange().GetStart() == i)
+                return i-1;
+
+            if (!wxRichTextCtrlIsWhitespace(text) /* && !text.empty() */)
                 i += direction;
             else
             {
@@ -1975,17 +2006,12 @@ wxRichTextCtrl::HitTest(const wxPoint& pt,
 
     int hit = ((wxRichTextCtrl*)this)->GetBuffer().HitTest(dc, pt2, *pos);
 
-    switch ( hit )
-    {
-        case wxRICHTEXT_HITTEST_BEFORE:
-            return wxTE_HT_BEFORE;
-
-        case wxRICHTEXT_HITTEST_AFTER:
-            return wxTE_HT_BEYOND;
-
-        case wxRICHTEXT_HITTEST_ON:
-            return wxTE_HT_ON_TEXT;
-    }
+    if ((hit & wxRICHTEXT_HITTEST_BEFORE) && (hit & wxRICHTEXT_HITTEST_OUTSIDE))
+        return wxTE_HT_BEFORE;
+    else if ((hit & wxRICHTEXT_HITTEST_AFTER) && (hit & wxRICHTEXT_HITTEST_OUTSIDE))
+        return wxTE_HT_BEYOND;
+    else if (hit & (wxRICHTEXT_HITTEST_BEFORE|wxRICHTEXT_HITTEST_AFTER))
+        return wxTE_HT_ON_TEXT;
 
     return wxTE_HT_UNKNOWN;
 }
@@ -2009,30 +2035,21 @@ void wxRichTextCtrl::DoSetValue(const wxString& value, int flags)
 {
     Clear();
 
-    // if the text is long enough, it's faster to just set it instead of first
-    // comparing it with the old one (chances are that it will be different
-    // anyhow, this comparison is there to avoid flicker for small single-line
-    // edit controls mostly)
-    if ( (value.length() > 0x400) || (value != GetValue()) )
+    if (!value.IsEmpty())
     {
+        // Remove empty paragraph
+        GetBuffer().Clear();
         DoWriteText(value);
 
         // for compatibility, don't move the cursor when doing SetValue()
         SetInsertionPoint(0);
     }
-    else // same text
+    else
     {
-        if ( flags & SetValue_SendEvent )
-        {
-            // still send an event for consistency
+        // still send an event for consistency
+        if (flags & SetValue_SendEvent)
             SendTextUpdatedEvent();
-        }
     }
-
-    // we should reset the modified flag even if the value didn't really change
-
-    // mark the control as being not dirty - we changed its text, not the
-    // user
     DiscardEdits();
 }
 
@@ -2481,8 +2498,14 @@ void wxRichTextCtrl::OnUpdateSelectAll(wxUpdateUIEvent& event)
     event.Enable(GetLastPosition() > 0);
 }
 
-void wxRichTextCtrl::OnContextMenu(wxContextMenuEvent& WXUNUSED(event))
+void wxRichTextCtrl::OnContextMenu(wxContextMenuEvent& event)
 {
+    if (event.GetEventObject() != this)
+    {
+        event.Skip();
+        return;
+    }
+
     if (!m_contextMenu)
     {
         m_contextMenu = new wxMenu;
@@ -2661,12 +2684,13 @@ void wxRichTextCtrl::PositionCaret()
     wxRect caretRect;
     if (GetCaretPositionForIndex(GetCaretPosition(), caretRect))
     {
-        wxPoint originalPt = caretRect.GetPosition();
-        wxPoint pt = GetPhysicalPoint(originalPt);
-        if (GetCaret()->GetPosition() != pt)
+        wxPoint newPt = caretRect.GetPosition();
+        wxSize newSz = caretRect.GetSize();
+        wxPoint pt = GetPhysicalPoint(newPt);
+        if (GetCaret()->GetPosition() != pt || GetCaret()->GetSize() != newSz)
         {
             GetCaret()->Move(pt);
-            GetCaret()->SetSize(caretRect.GetSize());
+            GetCaret()->SetSize(newSz);
         }
     }
 }
@@ -2684,6 +2708,10 @@ bool wxRichTextCtrl::GetCaretPositionForIndex(long position, wxRect& rect)
 
     if (GetBuffer().FindPosition(dc, position, pt, & height, m_caretAtLineStart))
     {
+        // Caret height can't be zero
+        if (height == 0)
+            height = dc.GetCharHeight();
+
         rect = wxRect(pt, wxSize(wxRICHTEXT_DEFAULT_CARET_WIDTH, height));
         return true;
     }
