@@ -6,7 +6,7 @@
 # Author:       Robin Dunn
 #
 # Created:      A long time ago, in a galaxy far, far away...
-# RCS-ID:       $Id: Main.py,v 1.213.2.11 2007/05/13 02:17:14 RD Exp $
+# RCS-ID:       $Id: Main.py 50244 2007-11-25 22:22:06Z RD $
 # Copyright:    (c) 1999 by Total Control Software
 # Licence:      wxWindows license
 #----------------------------------------------------------------------------
@@ -83,6 +83,8 @@ _treeList = [
         'SizedControls',
         'AUI_MDI',
         'TreeMixin',
+        'AdjustChannels',
+        'RendererNative',
         ]),
 
     # managed windows == things with a (optional) caption you can close
@@ -267,6 +269,7 @@ _treeList = [
 
     # Images
     ('Using Images', [
+        'AdjustChannels',
         'AlphaDrawing',
         'AnimateCtrl',
         'ArtProvider',
@@ -300,6 +303,7 @@ _treeList = [
         'OGL',
         'PrintFramework',
         'PseudoDC',
+        'RendererNative',
         'ShapedWindow',
         'Sound',
         'StandardPaths',
@@ -518,7 +522,7 @@ try:
                 # TODO: if this looks fine on Linux too, remove the Mac-specific case 
                 # and use this whenever OS != MSW.
                 self.StyleSetSpec(stc.STC_STYLE_DEFAULT, 
-                                  'fore:#000000,back:#FFFFFF,face:Courier')
+                                  'fore:#000000,back:#FFFFFF,face:Monaco')
             else:
                 self.StyleSetSpec(stc.STC_STYLE_DEFAULT, 
                                   'fore:#000000,back:#FFFFFF,face:Courier,size:9')
@@ -667,9 +671,9 @@ class DemoCodePanel(wx.Panel):
     def ActiveModuleChanged(self):
         self.LoadDemoSource(self.demoModules.GetSource())
         self.UpdateControlState()
-        self.mainFrame.Freeze()        
+        self.mainFrame.pnl.Freeze()        
         self.ReloadDemo()
-        self.mainFrame.Thaw()
+        self.mainFrame.pnl.Thaw()
 
         
     def LoadDemoSource(self, source):
@@ -845,6 +849,8 @@ def SearchDemo(name, keyword):
     fid = open(GetOriginalFilename(name), "rt")
     fullText = fid.read()
     fid.close()
+    if type(keyword) is unicode:
+        fullText = fullText.decode('iso8859-1')
     if fullText.find(keyword) >= 0:
         return True
 
@@ -901,9 +907,9 @@ class DemoModules:
     def LoadDict(self, modID):
         if self.name != __name__:
             source = self.modules[modID][1]
-            #description = self.modules[modID][3]
             description = self.modules[modID][2]
-
+            description = description.encode(sys.getfilesystemencoding())
+            
             try:
                 self.modules[modID][0] = {}
                 code = compile(source, description, "exec")        
@@ -1177,7 +1183,7 @@ class DemoTaskBarIcon(wx.TaskBarIcon):
 
 
     def OnTaskBarClose(self, evt):
-        self.frame.Close()
+        wx.CallAfter(self.frame.Close)
 
 
     def OnTaskBarChange(self, evt):
@@ -1206,9 +1212,14 @@ class wxPythonDemo(wx.Frame):
                           style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)
 
         self.SetMinSize((640,480))
+
+        # Use a panel under the AUI panes in order to work around a
+        # bug on PPC Macs
+        pnl = wx.Panel(self)
+        self.pnl = pnl
         
         self.mgr = wx.aui.AuiManager()
-        self.mgr.SetManagedWindow(self)
+        self.mgr.SetManagedWindow(pnl)
 
         self.loaded = False
         self.cwd = os.getcwd()
@@ -1244,7 +1255,7 @@ class wxPythonDemo(wx.Frame):
         self.ReadConfigurationFile()
         
         # Create a Notebook
-        self.nb = wx.Notebook(self, -1, style=wx.CLIP_CHILDREN)
+        self.nb = wx.Notebook(pnl, -1, style=wx.CLIP_CHILDREN)
         imgList = wx.ImageList(16, 16)
         for png in ["overview", "code", "demo"]:
             bmp = images.catalog[png].getBitmap()
@@ -1257,7 +1268,7 @@ class wxPythonDemo(wx.Frame):
         self.finddata.SetFlags(wx.FR_DOWN)
 
         # Create a TreeCtrl
-        leftPanel = wx.Panel(self, style=wx.TAB_TRAVERSAL|wx.CLIP_CHILDREN)
+        leftPanel = wx.Panel(pnl, style=wx.TAB_TRAVERSAL|wx.CLIP_CHILDREN)
         self.treeMap = {}
         self.searchItems = {}
         
@@ -1266,8 +1277,7 @@ class wxPythonDemo(wx.Frame):
         self.filter = wx.SearchCtrl(leftPanel, style=wx.TE_PROCESS_ENTER)
         self.filter.ShowCancelButton(True)
         self.filter.Bind(wx.EVT_TEXT, self.RecreateTree)
-        self.filter.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN,
-                         lambda e: self.filter.SetValue(''))
+        self.filter.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnSearchCancelBtn)
         self.filter.Bind(wx.EVT_TEXT_ENTER, self.OnSearch)
 
         searchMenu = wx.Menu()
@@ -1309,7 +1319,7 @@ class wxPythonDemo(wx.Frame):
 
 
         # Set up a log window
-        self.log = wx.TextCtrl(self, -1,
+        self.log = wx.TextCtrl(pnl, -1,
                               style = wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
         if wx.Platform == "__WXMAC__":
             self.log.MacCheckSpelling(False)
@@ -1466,7 +1476,10 @@ class wxPythonDemo(wx.Frame):
         menu = wx.Menu()
         findItem = wx.MenuItem(menu, -1, '&Find\tCtrl-F', 'Find in the Demo Code')
         findItem.SetBitmap(images.catalog['find'].getBitmap())
-        findNextItem = wx.MenuItem(menu, -1, 'Find &Next\tF3', 'Find Next')
+        if 'wxMac' not in wx.PlatformInfo:
+            findNextItem = wx.MenuItem(menu, -1, 'Find &Next\tF3', 'Find Next')
+        else:
+            findNextItem = wx.MenuItem(menu, -1, 'Find &Next\tCtrl-G', 'Find Next')
         findNextItem.SetBitmap(images.catalog['findnext'].getBitmap())
         menu.AppendItem(findItem)
         menu.AppendItem(findNextItem)
@@ -1479,8 +1492,9 @@ class wxPythonDemo(wx.Frame):
         inspToolItem = wx.MenuItem(menu, -1, 'Open &Widget Inspector\tF6',
                                    'A tool that lets you browse the live widgets and sizers in an application')
         inspToolItem.SetBitmap(images.catalog['inspect'].getBitmap())
-        menu.AppendItem(inspToolItem)        
-        menu.AppendSeparator()
+        menu.AppendItem(inspToolItem)
+        if 'wxMac' not in wx.PlatformInfo:
+            menu.AppendSeparator()
         helpItem = menu.Append(-1, '&About wxPython Demo', 'wxPython RULES!!!')
         wx.App.SetMacAboutMenuItemId(helpItem.GetId())
 
@@ -1626,6 +1640,11 @@ class wxPythonDemo(wx.Frame):
         self.RecreateTree()            
 
 
+    def OnSearchCancelBtn(self, event):
+        self.filter.SetValue('')
+        self.OnSearch()
+        
+
     def SetTreeModified(self, modified):
         item = self.tree.GetSelection()
         if modified:
@@ -1677,7 +1696,7 @@ class wxPythonDemo(wx.Frame):
     def LoadDemo(self, demoName):
         try:
             wx.BeginBusyCursor()
-            self.Freeze()
+            self.pnl.Freeze()
             
             os.chdir(self.cwd)
             self.ShutdownDemoModule()
@@ -1701,7 +1720,7 @@ class wxPythonDemo(wx.Frame):
                     self.UpdateNotebook(0)
         finally:
             wx.EndBusyCursor()
-            self.Thaw()
+            self.pnl.Thaw()
 
     #---------------------------------------------
     def LoadDemoSource(self):
@@ -1766,7 +1785,7 @@ class wxPythonDemo(wx.Frame):
     def UpdateNotebook(self, select = -1):
         nb = self.nb
         debug = False
-        self.Freeze()
+        self.pnl.Freeze()
         
         def UpdatePage(page, pageText):
             pageExists = False
@@ -1807,7 +1826,7 @@ class wxPythonDemo(wx.Frame):
         if select >= 0 and select < nb.GetPageCount():
             nb.SetSelection(select)
 
-        self.Thaw()
+        self.pnl.Thaw()
         
     #---------------------------------------------
     def SetOverview(self, name, text):
