@@ -2,7 +2,7 @@
 # Don't modify this file, modify the SWIG interface instead.
 
 """
-The wx.aui moduleis an Advanced User Interface library that aims to
+The wx.aui module is an Advanced User Interface library that aims to
 implement "cutting-edge" interface usability and design features so
 developers can quickly and easily create beautiful and usable
 application interfaces.
@@ -481,6 +481,10 @@ class AuiPaneInfo(object):
         """Movable(self, bool b=True) -> AuiPaneInfo"""
         return _aui.AuiPaneInfo_Movable(*args, **kwargs)
 
+    def DockFixed(*args, **kwargs):
+        """DockFixed(self, bool b=True) -> AuiPaneInfo"""
+        return _aui.AuiPaneInfo_DockFixed(*args, **kwargs)
+
     def Dockable(*args, **kwargs):
         """Dockable(self, bool b=True) -> AuiPaneInfo"""
         return _aui.AuiPaneInfo_Dockable(*args, **kwargs)
@@ -526,6 +530,7 @@ class AuiPaneInfo(object):
     optionActive = _aui.AuiPaneInfo_optionActive
     optionGripperTop = _aui.AuiPaneInfo_optionGripperTop
     optionMaximized = _aui.AuiPaneInfo_optionMaximized
+    optionDockFixed = _aui.AuiPaneInfo_optionDockFixed
     buttonClose = _aui.AuiPaneInfo_buttonClose
     buttonMaximize = _aui.AuiPaneInfo_buttonMaximize
     buttonMinimize = _aui.AuiPaneInfo_buttonMinimize
@@ -1031,6 +1036,7 @@ AUI_NB_WINDOWLIST_BUTTON = _aui.AUI_NB_WINDOWLIST_BUTTON
 AUI_NB_CLOSE_BUTTON = _aui.AUI_NB_CLOSE_BUTTON
 AUI_NB_CLOSE_ON_ACTIVE_TAB = _aui.AUI_NB_CLOSE_ON_ACTIVE_TAB
 AUI_NB_CLOSE_ON_ALL_TABS = _aui.AUI_NB_CLOSE_ON_ALL_TABS
+AUI_NB_MIDDLE_CLICK_CLOSE = _aui.AUI_NB_MIDDLE_CLICK_CLOSE
 AUI_NB_DEFAULT_STYLE = _aui.AUI_NB_DEFAULT_STYLE
 class AuiNotebookEvent(_core.NotifyEvent):
     """Proxy of C++ AuiNotebookEvent class"""
@@ -1316,6 +1322,14 @@ class AuiTabContainer(object):
         """SetTabOffset(self, size_t offset)"""
         return _aui.AuiTabContainer_SetTabOffset(*args, **kwargs)
 
+    def IsTabVisible(*args, **kwargs):
+        """IsTabVisible(self, int tabPage, int tabOffset, DC dc, Window wnd) -> bool"""
+        return _aui.AuiTabContainer_IsTabVisible(*args, **kwargs)
+
+    def MakeTabVisible(*args, **kwargs):
+        """MakeTabVisible(self, int tabPage, Window win)"""
+        return _aui.AuiTabContainer_MakeTabVisible(*args, **kwargs)
+
     ActivePage = property(GetActivePage,SetActivePage,doc="See `GetActivePage` and `SetActivePage`") 
     PageCount = property(GetPageCount,doc="See `GetPageCount`") 
     Pages = property(GetPages,doc="See `GetPages`") 
@@ -1335,6 +1349,10 @@ class AuiTabCtrl(_core.Control,AuiTabContainer):
 
     __swig_destroy__ = _aui.delete_AuiTabCtrl
     __del__ = lambda self : None;
+    def IsDragging(*args, **kwargs):
+        """IsDragging(self) -> bool"""
+        return _aui.AuiTabCtrl_IsDragging(*args, **kwargs)
+
 _aui.AuiTabCtrl_swigregister(AuiTabCtrl)
 
 class AuiNotebook(_core.Control):
@@ -1439,6 +1457,34 @@ class AuiNotebook(_core.Control):
         """GetAuiManager(self) -> AuiManager"""
         return _aui.AuiNotebook_GetAuiManager(*args, **kwargs)
 
+    def SetNormalFont(*args, **kwargs):
+        """SetNormalFont(self, Font font)"""
+        return _aui.AuiNotebook_SetNormalFont(*args, **kwargs)
+
+    def SetSelectedFont(*args, **kwargs):
+        """SetSelectedFont(self, Font font)"""
+        return _aui.AuiNotebook_SetSelectedFont(*args, **kwargs)
+
+    def SetMeasuringFont(*args, **kwargs):
+        """SetMeasuringFont(self, Font font)"""
+        return _aui.AuiNotebook_SetMeasuringFont(*args, **kwargs)
+
+    def GetTabCtrlHeight(*args, **kwargs):
+        """GetTabCtrlHeight(self) -> int"""
+        return _aui.AuiNotebook_GetTabCtrlHeight(*args, **kwargs)
+
+    def GetHeightForPageHeight(*args, **kwargs):
+        """GetHeightForPageHeight(self, int pageHeight) -> int"""
+        return _aui.AuiNotebook_GetHeightForPageHeight(*args, **kwargs)
+
+    def AdvanceSelection(*args, **kwargs):
+        """AdvanceSelection(self, bool forward=True)"""
+        return _aui.AuiNotebook_AdvanceSelection(*args, **kwargs)
+
+    def ShowWindowMenu(*args, **kwargs):
+        """ShowWindowMenu(self) -> bool"""
+        return _aui.AuiNotebook_ShowWindowMenu(*args, **kwargs)
+
     PageCount = property(GetPageCount,doc="See `GetPageCount`") 
     Selection = property(GetSelection,SetSelection,doc="See `GetSelection` and `SetSelection`") 
 _aui.AuiNotebook_swigregister(AuiNotebook)
@@ -1450,6 +1496,7 @@ def PreAuiNotebook(*args, **kwargs):
     return val
 
 wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE = _aui.wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE
+wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSED = _aui.wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSED
 wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED = _aui.wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED
 wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGING = _aui.wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGING
 wxEVT_COMMAND_AUINOTEBOOK_BUTTON = _aui.wxEVT_COMMAND_AUINOTEBOOK_BUTTON
@@ -1457,7 +1504,14 @@ wxEVT_COMMAND_AUINOTEBOOK_BEGIN_DRAG = _aui.wxEVT_COMMAND_AUINOTEBOOK_BEGIN_DRAG
 wxEVT_COMMAND_AUINOTEBOOK_END_DRAG = _aui.wxEVT_COMMAND_AUINOTEBOOK_END_DRAG
 wxEVT_COMMAND_AUINOTEBOOK_DRAG_MOTION = _aui.wxEVT_COMMAND_AUINOTEBOOK_DRAG_MOTION
 wxEVT_COMMAND_AUINOTEBOOK_ALLOW_DND = _aui.wxEVT_COMMAND_AUINOTEBOOK_ALLOW_DND
+wxEVT_COMMAND_AUINOTEBOOK_DRAG_DONE = _aui.wxEVT_COMMAND_AUINOTEBOOK_DRAG_DONE
+wxEVT_COMMAND_AUINOTEBOOK_BG_DCLICK = _aui.wxEVT_COMMAND_AUINOTEBOOK_BG_DCLICK
+wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_DOWN = _aui.wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_DOWN
+wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_UP = _aui.wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_UP
+wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_DOWN = _aui.wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_DOWN
+wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP = _aui.wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP
 EVT_AUINOTEBOOK_PAGE_CLOSE = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE, 1 )
+EVT_AUINOTEBOOK_PAGE_CLOSED = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSED, 1 )
 EVT_AUINOTEBOOK_PAGE_CHANGED = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, 1 )
 EVT_AUINOTEBOOK_PAGE_CHANGING = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGING, 1 )
 EVT_AUINOTEBOOK_BUTTON = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_BUTTON, 1 )
@@ -1465,6 +1519,12 @@ EVT_AUINOTEBOOK_BEGIN_DRAG = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_BEGIN_D
 EVT_AUINOTEBOOK_END_DRAG = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_END_DRAG, 1 )
 EVT_AUINOTEBOOK_DRAG_MOTION = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_DRAG_MOTION, 1 )
 EVT_AUINOTEBOOK_ALLOW_DND = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_ALLOW_DND, 1 )
+EVT_AUINOTEBOOK_DRAG_DONE = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_DRAG_DONE, 1 )
+EVT_AUINOTEBOOK_BG_DCLICK = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_BG_DCLICK, 1 )
+EVT_AUINOTEBOOK_TAB_MIDDLE_DOWN = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_DOWN, 1 )
+EVT_AUINOTEBOOK_TAB_MIDDLE_UP  = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_UP , 1 )
+EVT_AUINOTEBOOK_TAB_RIGHT_DOWN = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_DOWN, 1 )
+EVT_AUINOTEBOOK_TAB_RIGHT_UP = wx.PyEventBinder( wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP, 1 )
 
 class AuiMDIParentFrame(_windows.Frame):
     """Proxy of C++ AuiMDIParentFrame class"""
