@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     9-Dec-1999
-# RCS-ID:      $Id: buttons.py,v 1.20.2.1 2007/03/26 18:52:55 RD Exp $
+# RCS-ID:      $Id: buttons.py 49957 2007-11-14 22:43:24Z AG $
 # Copyright:   (c) 1999 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -64,7 +64,7 @@ class GenButton(wx.PyControl):
         if cstyle == 0:
             cstyle = wx.BORDER_NONE
         wx.PyControl.__init__(self, parent, id, pos, size, cstyle, validator, name)
-
+        
         self.up = True
         self.hasFocus = False
         self.style = style
@@ -90,6 +90,16 @@ class GenButton(wx.PyControl):
         self.Bind(wx.EVT_KEY_DOWN,         self.OnKeyDown)
         self.Bind(wx.EVT_KEY_UP,           self.OnKeyUp)
         self.Bind(wx.EVT_PAINT,            self.OnPaint)
+        self.Bind(wx.EVT_SIZE,             self.OnSize)
+        self.InitOtherEvents()
+
+    def InitOtherEvents(self):
+        """
+        Override in a subclass to initialize any other events that
+        need to be bound.  Added so __init__ doesn't need to be
+        overriden, which is complicated with multiple inheritance
+        """
+        pass
 
 
     def SetInitialSize(self, size=None):
@@ -276,6 +286,12 @@ class GenButton(wx.PyControl):
             self.DrawFocusIndicator(dc, width, height)
 
 
+    def OnSize(self, event):
+
+        self.Refresh()
+        event.Skip()
+
+
     def GetBackgroundBrush(self, dc):
         if self.up:
             colBg = self.GetBackgroundColour()
@@ -583,6 +599,15 @@ class GenBitmapTextToggleButton(__ToggleMixin, GenBitmapTextButton):
 
 class ThemedGenButton(GenButton):
     " A themed generic button, and base class for the other themed buttons "
+
+    def InitOtherEvents(self):
+        self.Bind(wx.EVT_ENTER_WINDOW, self.OnMouse)
+        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouse)
+        
+    def OnMouse(self, evt):
+        self.Refresh()
+        evt.Skip()
+
     def DrawBezel(self, dc, x1, y1, x2, y2):
         rect = wx.Rect(x1, y1, x2, y2)
         if self.up:
@@ -591,7 +616,11 @@ class ThemedGenButton(GenButton):
             state = wx.CONTROL_PRESSED
         if not self.IsEnabled():
             state = wx.CONTROL_DISABLED
+        pt = self.ScreenToClient(wx.GetMousePosition())
+        if self.GetClientRect().Contains(pt):
+            state = wx.CONTROL_CURRENT
         wx.RendererNative.Get().DrawPushButton(self, dc, rect, state)
+        
  
 class ThemedGenBitmapButton(ThemedGenButton, GenBitmapButton):
     """A themed generic bitmap button."""
