@@ -7,7 +7,7 @@
 #
 # Author:      Mike Fletcher
 #
-# RCS-ID:      $Id: filebrowsebutton.py,v 1.14.4.1 2007/02/19 21:17:21 RD Exp $
+# RCS-ID:      $Id: filebrowsebutton.py 50288 2007-11-28 00:18:44Z RD $
 # Copyright:   (c) 2000 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -180,12 +180,16 @@ class FileBrowseButton(wx.Panel):
         self.callCallback =  save
 
 
-    def Enable (self, value):
+    def Enable (self, value=True):
         """ Convenient enabling/disabling of entire control """
         self.label.Enable (value)
         self.textControl.Enable (value)
         return self.browseButton.Enable (value)
 
+    def Disable (self,):
+        """ Convenient disabling of entire control """
+        self.Enable(False)
+
     def GetLabel( self ):
         """ Retrieve the label's current text """
         return self.label.GetLabel()
