@@ -2,12 +2,14 @@
 # Purpose:      XRC editor, undo/redo module
 # Author:       Roman Rolinsky <rolinsky@mema.ucl.ac.be>
 # Created:      01.12.2002
-# RCS-ID:       $Id: undo.py,v 1.8.8.4 2007/05/08 02:34:22 RD Exp $
+# RCS-ID:       $Id: undo.py 49125 2007-10-10 23:15:03Z ROL $
 
 from globals import *
-from xxx import MakeXXXFromDOM
-from tree import *
-#from panel import *
+import view
+from component import Manager
+from model import Model
+
+undo_depth = 10                 # max number of undo remembered
 
 # Undo/redo classes
 class UndoManager:
@@ -15,71 +17,92 @@ class UndoManager:
     undo = []
     redo = []
     def RegisterUndo(self, undoObj):
+        TRACE('RegisterUndo: %s', undoObj.label)
         self.undo.append(undoObj)
-        for i in self.redo: i.destroy()
+        while len(self.undo) > undo_depth: self.undo.pop(0)
+        map(Undo.destroy, self.redo)
         self.redo = []
+        self.UpdateToolHelp()
+    def GetUndoLabel(self):
+        return self.undo[-1].label
+    def GetRedoLabel(self):
+        return self.redo[-1].label
     def Undo(self):
         undoObj = self.undo.pop()
         undoObj.undo()
         self.redo.append(undoObj)
-        g.frame.SetModified()
-        g.frame.SetStatusText('Undone')
+        view.frame.SetStatusText('Undone')
+        Presenter.undoSaved = True
+        self.UpdateToolHelp()
     def Redo(self):
         undoObj = self.redo.pop()
         undoObj.redo()
         self.undo.append(undoObj)
-        g.frame.SetModified()
-        g.frame.SetStatusText('Redone')
+        view.frame.SetStatusText('Redone')
+        Presenter.undoSaved = True
+        self.UpdateToolHelp()
     def Clear(self):
-        for i in self.undo: i.destroy()
+        map(Undo.destroy, self.undo)
         self.undo = []
-        for i in self.redo: i.destroy()
+        map(Undo.destroy, self.redo)
         self.redo = []
+        self.UpdateToolHelp()
     def CanUndo(self):
-        return not not self.undo
+        return bool(self.undo)
     def CanRedo(self):
-        return not not self.redo
+        return bool(self.redo)
+    def UpdateToolHelp(self):
+        if g.undoMan.CanUndo(): 
+            msg = 'Undo ' + self.GetUndoLabel()
+            view.frame.tb.SetToolShortHelp(wx.ID_UNDO, msg)
+            view.frame.tb.SetToolLongHelp(wx.ID_UNDO, msg)
+        if g.undoMan.CanRedo(): 
+            msg = 'Redo ' + self.GetRedoLabel()
+            view.frame.tb.SetToolShortHelp(wx.ID_REDO, msg)
+            view.frame.tb.SetToolLongHelp(wx.ID_REDO, msg)
 
-class UndoCutDelete:
-    def __init__(self, itemIndex, parent, elem):
-        self.itemIndex = itemIndex
-        self.parent = parent
-        self.elem = elem
+class Undo:
+    '''ABC for Undo*.'''
+    def redo(self):             # usually redo is same as undo
+        self.undo()
     def destroy(self):
-        if self.elem: self.elem.unlink()
-    def undo(self):
-        item = g.tree.InsertNode(g.tree.ItemAtFullIndex(self.itemIndex[:-1]),
-                                 self.parent, self.elem,
-                                 g.tree.ItemAtFullIndex(self.itemIndex))
-        # Scroll to show new item (!!! redundant?)
-        g.tree.EnsureVisible(item)
-        g.tree.SelectItem(item)
-        self.elem = None
-        # Update testWin if needed
-        if g.testWin and g.tree.IsHighlatable(item):
-            if g.conf.autoRefresh:
-                g.tree.needUpdate = True
-                g.tree.pendingHighLight = item
-            else:
-                g.tree.pendingHighLight = None
-    def redo(self):
-        item = g.tree.ItemAtFullIndex(self.itemIndex)
-        # Delete testWin?
-        if g.testWin:
-            # If deleting top-level item, delete testWin
-            if item == g.testWin.item:
-                g.testWin.Destroy()
-                g.testWin = None
-            else:
-                # Remove highlight, update testWin
-                if g.testWin.highLight:
-                    g.testWin.highLight.Remove()
-                g.tree.needUpdate = True
-        self.elem = g.tree.RemoveLeaf(item)
-        g.tree.UnselectAll()
-        g.panel.Clear()
+        pass
 
-class UndoPasteCreate:
+class UndoCutDelete(Undo):
+    label = 'cut/delete'
+    def __init__(self, itemIndex, node):
+        self.itemIndex = itemIndex
+        self.node = node
+        self.states = view.tree.GetFullState()
+    def destroy(self):
+        if self.node: self.node.unlink()
+        self.states = None
+    def undo(self):
+        # Updating DOM. Find parent node first
+        parentItem = view.tree.ItemAtFullIndex(self.itemIndex[:-1])
+        parentNode = view.tree.GetPyData(parentItem)
+        parentComp = Manager.getNodeComp(parentNode)
+        nextItem = view.tree.ItemAtFullIndex(self.itemIndex)
+        if nextItem:
+            nextNode = parentComp.getTreeOrImplicitNode(view.tree.GetPyData(nextItem))
+        else:
+            nextNode = None
+        # Insert before next
+        parentNode.insertBefore(self.node, nextNode)
+        # Update tree and presenter
+        view.tree.Flush()
+        view.tree.SetFullState(self.states)
+        item = view.tree.ItemAtFullIndex(self.itemIndex)
+        view.tree.EnsureVisible(item)
+        # This will generate events
+        view.tree.SelectItem(item)
+    def redo(self):
+        item = view.tree.ItemAtFullIndex(self.itemIndex)
+        view.tree.SelectItem(item)
+        self.node = Presenter.delete(item)
+
+class UndoPasteCreate(Undo):
+    label = 'paste/create'
     def __init__(self, itemParent, parent, item, selected):
         self.itemParentIndex = g.tree.ItemFullIndex(itemParent)
         self.parent = parent
@@ -121,7 +144,8 @@ class UndoPasteCreate:
             else:
                 g.tree.pendingHighLight = None
 
-class UndoReplace:
+class UndoReplace(Undo):
+    label = 'replace'
     def __init__(self, item):
         self.itemIndex = g.tree.ItemFullIndex(item)
         #self.xxx = g.tree.GetPyData(item)
@@ -165,22 +189,26 @@ class UndoReplace:
     def redo(self):
         return
 
-class UndoMove:
+class UndoMove(Undo):
+    label = 'move'
     def __init__(self, oldParent, oldIndex, newParent, newIndex):
-        self.oldParent = oldParent
+        # Store indexes because items can be invalid already
+        self.oldParentIndex = g.tree.ItemFullIndex(oldParent)
         self.oldIndex = oldIndex
-        self.newParent = newParent
+        self.newParentIndex = g.tree.ItemFullIndex(newParent)
         self.newIndex = newIndex
     def destroy(self):
         pass
     def undo(self):
-        item = g.tree.GetFirstChild(self.newParent)[0]
+        oldParent = g.tree.ItemAtFullIndex(self.oldParentIndex)
+        newParent = g.tree.ItemAtFullIndex(self.newParentIndex)
+        item = g.tree.GetFirstChild(newParent)[0]
         for i in range(self.newIndex): item = g.tree.GetNextSibling(item)
         elem = g.tree.RemoveLeaf(item)
-        nextItem = g.tree.GetFirstChild(self.oldParent)[0]
+        nextItem = g.tree.GetFirstChild(oldParent)[0]
         for i in range(self.oldIndex): nextItem = g.tree.GetNextSibling(nextItem) 
 
-        parent = g.tree.GetPyData(self.oldParent).treeObject()
+        parent = g.tree.GetPyData(oldParent).treeObject()
 
         # Check parent and child relationships.
         # If parent is sizer or notebook, child is of wrong class or
@@ -208,7 +236,7 @@ class UndoMove:
             pageElem.appendChild(elem)
             elem = pageElem
 
-        selected = g.tree.InsertNode(self.oldParent, parent, elem, nextItem)
+        selected = g.tree.InsertNode(oldParent, parent, elem, nextItem)
         g.tree.EnsureVisible(selected)
         # Highlight is outdated
         if g.testWin and g.testWin.highLight:
@@ -216,11 +244,13 @@ class UndoMove:
             g.tree.needUpdate = True
         g.tree.SelectItem(selected)
     def redo(self):
-        item = g.tree.GetFirstChild(self.oldParent)[0]
+        oldParent = g.tree.ItemAtFullIndex(self.oldParentIndex)
+        newParent = g.tree.ItemAtFullIndex(self.newParentIndex)
+        item = g.tree.GetFirstChild(oldParent)[0]
         for i in range(self.oldIndex): item = g.tree.GetNextSibling(item)
         elem = g.tree.RemoveLeaf(item)
 
-        parent = g.tree.GetPyData(self.newParent).treeObject()
+        parent = g.tree.GetPyData(newParent).treeObject()
 
         # Check parent and child relationships.
         # If parent is sizer or notebook, child is of wrong class or
@@ -248,9 +278,9 @@ class UndoMove:
             pageElem.appendChild(elem)
             elem = pageElem
 
-        nextItem = g.tree.GetFirstChild(self.newParent)[0]
+        nextItem = g.tree.GetFirstChild(newParent)[0]
         for i in range(self.newIndex): nextItem = g.tree.GetNextSibling(nextItem) 
-        selected = g.tree.InsertNode(self.newParent, parent, elem, nextItem)
+        selected = g.tree.InsertNode(newParent, parent, elem, nextItem)
         g.tree.EnsureVisible(selected)
         # Highlight is outdated
         if g.testWin and g.testWin.highLight:
@@ -258,34 +288,52 @@ class UndoMove:
             g.tree.needUpdate = True
         g.tree.SelectItem(selected)
 
-class UndoEdit:
-    def __init__(self):
-        self.pages = map(ParamPage.GetState, g.panel.pages)
-        self.selectedIndex = g.tree.ItemFullIndex(g.tree.GetSelection())
-    def destroy(self):
-        pass
-    # Update test view
-    def update(self, selected):
-        g.tree.Apply(g.tree.GetPyData(selected), selected)
-        # Update view
-        if g.testWin:
-            if g.testWin.highLight:
-                g.testWin.highLight.Remove()
-            g.tree.pendingHighLight = selected
-        if g.testWin:
-            g.tree.needUpdate = True
+
+class UndoEdit(Undo):
+    '''Undo class for using in AttributePanel.'''
+    label = 'edit'
+    def __init__(self, item, page):
+        self.index = view.tree.ItemFullIndex(item)
+        self.page = page
+        panel = view.panel.nb.GetPage(page).panel
+        self.values = panel.GetValues()
     def undo(self):
-        # Restore selection
-        selected = g.tree.ItemAtFullIndex(self.selectedIndex)
-        if selected != g.tree.GetSelection():
-            g.tree.SelectItem(selected)
-        # Save current state for redo
-        map(ParamPage.SaveState, g.panel.pages)
-        pages = map(ParamPage.GetState, g.panel.pages)
-        if self.pages:
-            map(ParamPage.SetState, g.panel.pages, self.pages)
-        self.pages = pages
-        self.update(selected)
+#        import pdb;pdb.set_trace()
+        # Go back to the item if needed
+        item = view.tree.GetSelection()
+        if not item or self.index != view.tree.ItemFullIndex(item):
+            Presenter.unselect()
+            undoItem = view.tree.ItemAtFullIndex(self.index)
+            if item != undoItem:
+                view.tree.SelectItem(view.tree.ItemAtFullIndex(self.index))
+                wx.Yield()          # Refresh panel
+        panel = view.panel.nb.GetPage(self.page).panel
+        values = panel.GetValues()
+        panel.SetValues(self.values)
+        self.values = values
+        # This will not generate events so we have to update the undo object
+        view.panel.nb.ChangeSelection(self.page)
+        Presenter.createUndoEdit(item, self.page)
+
+class UndoGlobal(Undo):
+    label = 'global'
+    def __init__(self):
+        self.mainNode = Model.mainNode.cloneNode(True)
+        self.states = view.tree.GetFullState()
+    def destroy(self):
+        self.mainNode.unlink()
+    def undo(self):
+        # Exchange
+        Model.mainNode,self.mainNode = \
+            self.mainNode,Model.dom.replaceChild(self.mainNode, Model.mainNode)
+        # Replace testElem
+        Model.testElem = Model.mainNode.childNodes[0]
+        states = view.tree.GetFullState()
+        Presenter.unselect()
+        view.tree.Flush()
+        view.tree.SetFullState(self.states)
+        self.states = states
     def redo(self):
         self.undo()
-        self.update(g.tree.GetSelection())
+        Presenter.unselect()
+
